#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : runware
# @Time         : 2025/10/9 16:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://runware.ai/models#video-featured

from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.llm.clients import AsyncClient
from meutils.notice.feishu import send_message_for_images
from meutils.io.files_utils import to_url, to_url_fal
from meutils.schemas.image_types import ImageRequest, RecraftImageRequest, ImagesResponse
from openai import APIStatusError

from meutils.apis.translator import deeplx


async def generate(request: ImageRequest, api_key: str, base_url: Optional[str] = None):
    payload = {
        **request.model_dump(exclude_none=True, exclude={"extra_fields", "aspect_ratio"}),
        **(request.extra_fields or {})
    }
    payload = [
        {
            "taskUUID": str(uuid.uuid4()),
            "taskType": "imageInference",
            "model": request.model,
            "positivePrompt": await deeplx.llm_translate(request.prompt),

            "numberResults": request.n,
            # "width": 1024,
            # "height": 1024,  # aspect_ratio todo 如何映射到 width/height
            "aspectRatio": request.aspect_ratio,

            "includeCost": True,
            "outputFormat": "PNG",
            "outputType": [
                "URL"
            ],

            **payload
        }
    ]

    if image_urls := request.image_urls:
        if not image_urls[0].startswith("http"):  # 转换为url
            image_urls = await to_url_fal(image_urls, content_type="image/png")

        payload[0]["referenceImages"] = image_urls

    if request.model in {"runware:100@1", "google:4@1", "google:4@2"}:  # flux
        payload[0]["width"], payload[0]["height"] = map(int, request.size.split("x"))

    elif aspect_ratio_mapping := await redis_aclient.get(f"runware:{request.model}"):
        aspect_ratio_mapping = aspect_ratio_mapping.decode()
        logger.debug(aspect_ratio_mapping)

        aspect_ratio_mapping = json.loads(aspect_ratio_mapping)

        if request.resolution:
            request.aspect_ratio = f"{request.aspect_ratio} {request.resolution.upper()}"
            # 1:1 1K

        size = (
                aspect_ratio_mapping.get(request.aspect_ratio)
                or aspect_ratio_mapping.get(request.aspect_ratio)
                or "1024x1024"
        )

        if size and isinstance(size, list):
            size = size[-1]  # 取最高清

        payload[0]["width"], payload[0]["height"] = map(int, size.split("x"))

    logger.debug(bjson(payload))
    try:
        client = AsyncClient(base_url="https://api.runware.ai/v1", api_key=api_key, timeout=300)
        response = await client.post(
            "/",
            body=payload,
            cast_to=object
        )

        if data := response.get("data"):
            for d in data:
                d["url"] = d["imageURL"]

            payload[0]["data"] = data
            send_message_for_images(payload[0], title=__name__)  # 存图片

            return ImagesResponse(data=data)
        elif errors := response.get("errors"):
            raise Exception(errors[0].get("message"))

    except APIStatusError as e:
        if (errors := e.response.json().get("errors")):
            logger.debug(bjson(errors))
            if (
                    any(i.lower() in str(errors).lower() for i in {"width/height", "unsupportedDimensions"})
                    and not await redis_aclient.get(f"runware:{request.model}")
                    and (v := errors[0].get("allowedValues", ""))

            ):
                logger.debug(v)
                await redis_aclient.set(f"runware:{request.model}", json.dumps(v), ex=30 * 24 * 3600)

        raise e


if __name__ == '__main__':
    model = "google:2@1"
    model = "runware:100@1"
    # model = "google:2@3"
    model = "bfl:3@1"
    # model = "bfl:4@1"

    # model = "google:4@1" # gemini
    model = "google:4@2"
    model = "google:4@2_4k"

    # prompt = "一个裸体少女"
    prompt = "a cat"
    request = ImageRequest(model=model, prompt=prompt, aspect_ratio="1:2")

    data = {
        "model": model,
        "prompt": "Turn this person into a character figure, the character sits on a Siberian tiger, the tiger's hair is black and yellow, the tiger's hair is very dense. Behind it, place a PS5 box with the character’s image and the game's name \"Black Myth: Zhong Kui\" printed on it, and a computer showing the Blender modeling process on its screen. In front of the box, add a round plastic base with the character figure standing on it. This figurine is made of PVC, and set the scene indoors.   (flux-kontext-pro)",
        "negative_prompt": "",
        "n": 1,
        "response_format": "url",
        "size": "1152x2048",
        "num_inference_steps": 20,
        "seed": None,
        "aspect_ratio": "9:16"
    }
    request = ImageRequest(**data)
    request = ImageRequest(
        model=model,
        prompt="将鸭子放在女人的t恤上",
        aspect_ratio= "9:16",
        size="1024x10240",
        image=[
            "https://v3.fal.media/files/penguin/XoW0qavfF-ahg-jX4BMyL_image.webp",
            "https://v3.fal.media/files/tiger/bml6YA7DWJXOigadvxk75_image.webp"
        ]
    )
    #
    # data = {"model": model, "prompt": "Generate a professional studio portrait of a Asian woman. ", "image": [],
    #  "size": "16:9", "aspect_ratio": "16:911"}
    # data = {"model": model, "prompt": "Generate a professional studio portrait of a Asian woman. ", "image": [],
    #         "size": "1024x1024"}
    # request = ImageRequest(**data)

    logger.debug(request)
    print(request.size)

    arun(generate(request, api_key="LYIVuIlPmO83ptihAu9mHgRZ82MH5nCx"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : usage
# @Time         : 2025/10/12 13:20
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.schemas.openai_types import CompletionUsage, ChatCompletion

d = {"id": "cgt-20251012131916-zdg8q", "model": "doubao-seedance-1-0-lite-t2v-250428", "status": "succeeded",
     "content": {
         "video_url": "https://ark-content-generation-cn-beijing.tos-cn-beijing.volces.com/doubao-seedance-1-0-lite-t2v/02176024635702100000000000000000000ffffac15ae33f8ddbe.mp4?X-Tos-Algorithm=TOS4-HMAC-SHA256&X-Tos-Credential=AKLTYWJkZTExNjA1ZDUyNDc3YzhjNTM5OGIyNjBhNDcyOTQ%2F20251012%2Fcn-beijing%2Ftos%2Frequest&X-Tos-Date=20251012T051950Z&X-Tos-Expires=86400&X-Tos-Signature=604ba900d31722e95c8b129a95d71ef2b0c9717c719d549a26c904aa19cca6be&X-Tos-SignedHeaders=host"},
     "usage": {"completion_tokens": 103818, "total_tokens": 103818}, "created_at": 1760246356, "updated_at": 1760246390,
     "seed": 8961, "resolution": "720p", "duration": 5, "ratio": "16:9", "framespersecond": 24}



CompletionUsage(**d['usage'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : upload
# @Time         : 2025/4/21 16:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
import asyncio
import httpx
import os
import mimetypes
import json
import sys
from pathlib import Path

mime_type = mimetypes.guess_type("x.html")[0] or "application/octet-stream"

print(mime_type)

num_bytes = len(Path("x.html").read_bytes())

headers = {
    "X-Goog-Upload-Protocol": "resumable",
    "X-Goog-Upload-Command": "start",
    "X-Goog-Upload-Header-Content-Length": str(num_bytes),
    "X-Goog-Upload-Header-Content-Type": mime_type,
    "Content-Type": "application/json",



}
# payload = {'file': {'display_name': "TEXT"}}
payload = {}
base_url = "https://all.chatfire.cc/genai"

client = httpx.Client(base_url=base_url, headers=headers)

response = client.post("/upload/v1beta/files", params={"key": os.getenv("GOOGLE_API_KEY")}, json=payload)

print(dict(response.headers))
upload_url = response.headers.get("x-goog-upload-url")
print(upload_url)

headers = {"X-Goog-Upload-Offset": "0",
    "X-Goog-Upload-Command": "upload, finalize", **headers}

response = httpx.post(url=upload_url, headers=headers, content=Path("x.html").read_bytes())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatglm_types
# @Time         : 2024/3/11 20:10
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
# 💡：用于表示新的想法或者重要的信息。
# ✔️：用于表示成功的执行或者完成的任务。
# > ✔️ The code executed successfully!
# 📊：用于表示数据或者统计结果。
# 📈：用于表示增长或者进步。
# 🎯：用于表示达到的目标或者预期的结果。
# 🚀：用于表示速度、效率或者进步。

from meutils.pipe import *

BASE_URL = "https://chatglm.cn/chatglm/backend-api/v1"
VIDEO_BASE_URL = "https://chatglm.cn/chatglm/video-api/v1"


class Part(BaseModel):
    """

    meta_data
        "metadata_list": [{
            "type": "webpage",
            "title": "南京天气预报,南京7天天气预报,南京15天天气预报,南京天气查询",
            "url": "http://www.weather.com.cn/weather/101190101.shtml",
            "text": "Web 结果1 天前 · 南京天气预报,南京7天天气预报,南京15天天气预报,南京天气查询. 预报. 全国 > 江苏 > 南京 > 城区. 18:00更新 | 数据来源 中央气象台. 今天. 7天. 8-15天. 40天. 雷达图. 2日（今天） 晴. 0℃. 3-4级. 3日（明天） 多云. 14℃ / 3℃. <3级转3-4级. 4日（后天） 阵雨转中雨. 13℃ / 7℃. <3级. 5日（周二） 阵雨转阴. 13℃ / 5℃. 4-5级. 6日（周 …   ",
            "pub_date": "1970-01-01T00:00:00.0000000"
        }]
    """
    id: str
    logic_id: str = ''
    role: str
    content: List[Dict[str, Any]]
    model: str
    recipient: str = ''
    created_at: str
    meta_data: dict = {}

    status: str

    # 预处理
    event: str = ''  # 类型
    markdown_data: str = ''

    def __init__(self, **data):
        super().__init__(**data)

        # self.event =
        # logger.debug(f"{self.status}: {self.content}")
        # logger.debug(
        #     f"""{self.status}: {self.content}\n{self.content and self.content[0].get("type")} \n {self.meta_data}""")
        # tool_calls image browser_result quote_result system_error
        if self.status == "finish" and self.content:
            content_type = self.content[0].get("type")

            # tool_calls
            if content_type == "tool_calls":
                _ = self.content[0].get("tool_calls", {})
                if "mclick" not in str(_):
                    self.markdown_data = f"""\n```{_.get("name")}\n{_}\n```\n"""

            if content_type == "quote_result" and self.meta_data:
                # logger.debug(self.meta_data)
                for metadata in self.meta_data.get("metadata_list", []):
                    if metadata.get("type") == "webpage":
                        self.markdown_data += f"[🔗{metadata.get('title')}]({metadata.get('url')})\n\n"

            # code
            if content_type == "code":
                code = self.content[0].get("code", "")
                self.markdown_data += f"""\n```{self.meta_data.get("toolCallRecipient", "python")}\n{code}\n```\n"""

            if content_type == "execution_output":  # todo: 展示块
                _ = self.content[0]
                self.markdown_data = f"\n```{content_type}\n{_}\n```\n{self.content[0].get('content')}\n"

            # image
            if content_type == "image" and self.status == "finish":

                images = self.content[0].get("image", [])
                for image in images:
                    self.markdown_data += f"![image]({image['image_url']})\n\n"

        # text
        if self.content and self.content[0].get("type") == "text":
            # self.markdown_data = f"""<text>{self.content[0].get("text", "")}"""
            self.markdown_data = f"""{self.content[0].get("text", "")}"""


class Data(BaseModel):
    """
    {
        "id": "65eef45f3901fe6e0bb7153b",
        "conversation_id": "65eef45e3901fe6e0bb7153a",
        "assistant_id": "65940acff94777010aa6b796",
        "parts": [
            {
                "id": "65eef45f3901fe6e0bb7153b",
                "logic_id": "62a2d941-43ba-4a00-9933-4f2a18979201",
                "role": "assistant",
                "content": [
                    {
                        "type": "text",
                        "text": "这是我为您创作的可爱猫咪图画，希望您喜欢。",
                        "status": "finish"
                    }
                ],
                "model": "chatglm-all-tools",
                "recipient": "all",
                "created_at": "2024-03-11 20:09:03",
                "meta_data": {
                    "toolCallRecipient": null
                },
                "status": "finish"  #
            }
        ],
        "created_at": "2024-03-11 20:09:03",
        "meta_data": {},
        "status": "finish",
        "last_error": {}
    }

    """
    id: str = "65940acff94777010aa6b796"  # chatglm4
    conversation_id: str
    assistant_id: str
    parts: List[Part]
    created_at: str
    status: str
    last_error: dict


"https://chatglm.cn/chatglm/video-api/v1/chat"

EXAMPLES = [
    {
        "prompt": "一艘巨大的古典帆船在巨浪的海面上行驶，灰蒙蒙的夜晚，月光照出蓝色的光影，风浪显得气氛很紧张",
        "conversation_id": "",
        "advanced_parameter_extra": {
            "video_style": "电影感",
            "emotional_atmosphere": "凄凉寂寞",
            "mirror_mode": "推近"
        }
    },

    {
        "prompt": "跳动起来",
        "conversation_id": "",
        "source_list": ["66a3379d3497367b9914de49"]  # 66a3379d3497367b9914de49 66a76a300603e53bccba4a1b
    }
]

"""
卡通3D
黑白老照片
油画
电影感

温馨和谐
生动活泼
紧张刺激
凄凉寂寞

水平
垂直
推近
拉远

"""


class Parameter(BaseModel):
    video_style: str = ''
    emotional_atmosphere: str = ''
    mirror_mode: str = ''


class VideoRequest(BaseModel):
    prompt: str
    conversation_id: str = ''
    advanced_parameter_extra: Parameter = {}
    source_list: Optional[list] = None  # 视频 "66a3373d57c37b00f049f7e5"

    class Config:
        json_schema_extra = {
            "examples": EXAMPLES
        }

# "https://chatglm.cn/chatglm/video-api/v1/static/composite_video" 加配音

# 66a6f2890603e53bccb9aa98



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : tune
# @Time         : 2024/9/20 13:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.apis.proxy.ips import get_one_proxy
from meutils.decorators.retry import retrying
from meutils.str_utils.regular_expression import remove_date_suffix
from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import aget_spreadsheet_values, get_next_token_for_polling

from meutils.llm.utils import oneturn2multiturn
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, ChatCompletionRequest, CompletionUsage

BASE_URL = "https://chat.tune.app"
FEISHU_URL_VIP = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=gCrlN4"
FEISHU_URL_API = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=9HwQtX"
"https://chat.tune.app/api/models"
# r = requests.get("https://chat.tune.app/tune-api/appConfig")
# r = requests.get("https://chat.tune.app/api/guestLogin")

# https://chat.tune.app/api/models?noCache=true

send_message = partial(
    _send_message,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e0db85db-0daf-4250-9131-a98d19b909a9",
    title=__name__
)

# data = httpx.get("https://chat.tune.app/api/models?noCache=true", timeout=30).json()
# models = jsonpath.jsonpath(data, expr='$..id')
#
# DEFAULT_MODEL = "openai/gpt-4o-mini"  # "kaushikaakash04/tune-blob"
# MODELS = {model.split('/')[-1]: model for model in models}

# logger.debug(bjson(MODELS))


DEFAULT_MODEL = "openai/gpt-4o-mini"
MODELS = {
    "o1-mini": "openai/o1-mini",
    "o1-preview": "openai/o1-preview",
    "o1-mini-2024-09-12": "openai/o1-mini",
    "o1-preview-2024-09-12": "openai/o1-preview",
    "gpt-4o": "openai/gpt-4o",
    "gpt-4o-mini": "openai/gpt-4o-mini",

    "claude-3.5-sonnet": "anthropic/claude-3.5-sonnet",
    "claude-3.5-haiku": "anthropic/claude-3.5-haiku",
    "claude-3-haiku": "anthropic/claude-3-haiku",

    "claude-3.5-sonnet-20240620": "anthropic/claude-3.5-sonnet",
    "claude-3.5-sonnet-20241022": "anthropic/claude-3.5-sonnet",
    "claude-3.5-haiku-20241022": "anthropic/claude-3.5-haiku",

    "gemini-flash-1.5-8b": "google/gemini-flash-1.5-8b",

    "tune-blob": "kaushikaakash04/tune-blob",
    "llama-3.1-8b-instruct": "meta/llama-3.1-8b-instruct",
    "pixtral-12B-2409": "mistral/pixtral-12B-2409",
    "qwen-2.5-coder-32b": "qwen/qwen-2.5-coder-32b",
    "llama-3.1-70b-instruct": "meta/llama-3.1-70b-instruct",
    "llama-3.2-90b-vision": "meta/llama-3.2-90b-vision",
    "llama3.1-euryale-70b": "sao10k/llama3.1-euryale-70b",
    "qwen-2.5-72b": "qwen/qwen-2.5-72b",
    "openrouter-goliath-120b-4k": "rohan/openrouter-goliath-120b-4k",
    "mistral-large": "mistral/mistral-large",
    "llama-3.1-405b-instruct": "meta/llama-3.1-405b-instruct",
    "mixtral-8x7b-instruct": "mistral/mixtral-8x7b-instruct",
    "hermes-3-llama-3.1-405b": "nousresearch/hermes-3-llama-3.1-405b",
    "tune-wizardlm-2-8x22b": "rohan/tune-wizardlm-2-8x22b",

    "tune-mythomax-l2-13b": "rohan/tune-mythomax-l2-13b",
    "qwen-2-vl-72b": "qwen/qwen-2-vl-72b",
}


#
# html_content = httpx.get(url).text
#
# # 正则表达式匹配以 "/_next/static/chunks/7116-" 开头的 JS 文件
# pattern = r'(/_next/static/chunks/7116-[^"]+\.js)'
#
# # 使用 re.findall() 找到所有匹配项
# matches = re.findall(pattern, html_content)


#
# "/_next/static/chunks/7116-aed224a0caaab94c.js"
#
# # 打印结果
# for match in matches:
#     print(match)


@alru_cache(ttl=1000)
@retrying(predicate=lambda r: r is None)
async def get_access_token():
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=30) as client:
        response = await client.get("/api/guestLogin")

        logger.debug(response.status_code)
        logger.debug(response.text)

        if response.is_success:
            return response.json()["accessToken"]


@alru_cache(ttl=60)
@retrying(predicate=lambda r: r is None)
async def create_conversation_id(token: str):
    headers = {
        "authorization": token
    }
    conversation_id = str(uuid.uuid4())  # shortuuid.random()
    # conversation_id = "af306c40-8f85-47a7-a027-185da084c6cc"
    params = {
        "conversation_id": conversation_id,
        "organization_id": "undefined",
        "model": "anthropic/claude-3.5-sonnet",  # "kaushikaakash04/tune-blob",
        "currency": "USD"
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.post("/api/new", params=params)
        logger.debug(response.status_code)
        logger.debug(response.text)
        return conversation_id

        # response = await client.get(f"/?id={conversation_id}", headers=headers)
        # html_content = response.text
        # pattern = r'(/_next/static/chunks/7116-[^"]+\.js)'
        # js = re.findall(pattern, html_content)[-1]
        #
        # logger.debug(js)
        #
        # response = await client.get(js)
        #
        # logger.debug(response.status_code)
        # next_action = response.text.split(',A=(0,a.$)("')[1][:40]
        # logger.debug(next_action)  # 2bc738a7215e149dbd4601a440f3b6df45089338 过期时间
        #
        # headers = {
        #     'next-action': next_action,
        #     'Cookie': f"AccessToken={token}",
        #
        #     'content-type': 'text/plain;charset=UTF-8',
        # }
        # payload = "[]"
        # response = await client.post(f"/?id={conversation_id}", headers=headers, content=payload)
        #
        # logger.debug(response.status_code)
        # logger.debug(response.text)
        #
        # return conversation_id


@retrying(max_retries=3)
async def create(request: ChatCompletionRequest, token: Optional[str] = None, vip: Optional[bool] = False):
    request.model = remove_date_suffix(request.model)

    if vip:
        token = await get_next_token_for_polling(feishu_url=FEISHU_URL_VIP)

    token = token or await get_access_token()
    conversation_id = await create_conversation_id(token)

    logger.debug(conversation_id)

    use_search = False
    if request.messages[0].get('role') != 'system':  # 还原系统信息
        request.messages.insert(0, {'role': 'system', 'content': f'You are {request.model}'})

    if request.model.startswith("net-") or request.model.endswith("all"):
        request.model = "kaushikaakash04/tune-blob"
        use_search = True
    else:
        request.model = MODELS.get(request.model, DEFAULT_MODEL)
    logger.debug(request)

    headers = {
        "authorization": token,
        # 'Cookie': f"AccessToken={token};",
        "content-type": "text/plain;charset=UTF-8",

    }
    params = {
        # "organization_id": "undefined",
        # "organization_id": "eb0fb996-2317-467b-9847-15f6c40000b7",
        "retry": 2,
    }
    payload = {
        # "query": request.last_content,
        "query": oneturn2multiturn(request.messages),
        # "images": request.urls,  # todo: 兼容base64

        "conversation_id": conversation_id,
        "model_id": request.model,  # "kaushikaakash04/tune-blob"
        "browseWeb": use_search,
        "attachement": "",
        "attachment_name": "",
        # "messageId": "4a33e497-efb7-4d8f-ae45-9aa7d2c1c5af1726811555410",
        # "prevMessageId": "4a33e497-efb7-4d8f-ae45-9aa7d2c1c5af1726811555410",

        "check": "286600"
    }

    yield "\n"  # 提升首字速度
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=300) as client:
        async with client.stream("POST", "/api/prompt", json=payload, params=params) as response:
            logger.debug(response.status_code)
            # logger.debug(response.text)

            async for chunk in response.aiter_lines():
                # logger.debug(chunk)

                # if chunk == '{"value":""}':
                #     async for _chunk in create(request, token, vip):
                #         yield _chunk
                #     return

                if chunk and chunk.startswith("{"):
                    chunk = (
                        chunk.replace("Blob", "OpenAI")
                        .replace("TuneAI", "OpenAI")
                        .replace("TuneStudio", "OpenAI")
                        .replace("Tune", "OpenAI")
                        .replace("https://studio.tune.app", "https://openai.com")
                        .replace("https://tunehq.ai", "https://openai.com")
                        .replace("https://chat.tune.app", "https://openai.com")
                    )

                    try:
                        chunk = json.loads(chunk)
                        chunk = chunk.get('value', "")
                        yield chunk
                        # break

                    except Exception as e:
                        _ = f"{e}\n{chunk}"
                        logger.error(_)
                        send_message(_)
                        yield ""
                elif chunk.strip():
                    logger.debug(chunk)

            # {"value": "完成"}


if __name__ == '__main__':
    # arun(get_access_token())
    # arun(create_conversation_id(None))

    # model = "claude-3.5-sonnet"
    # model = "net-anthropic/claude-3.5-sonnet"
    # model = "all"

    model = "kaushikaakash04/tune-blob"
    # model = "openai/o1-mini"
    # model = "o1-mini-0609"

    # model = "openai/gpt-4o-mini"

    url = "https://d2e931syjhr5o9.cloudfront.net/bb5cc8e4-5cc2-45c5-8307-585b7ec6eada_1731634172097.png"

    request = ChatCompletionRequest(model=model, messages=[
        {'role': 'user', 'content': '你的知识库截止的几月份'},
        # {
        #     'role': 'user', 'content': [
        #     {"type": "text", "text": "解释下这张图片"},
        #     {"type": "image_url", "image_url": {"url": url}},
        # ]
        # }
    ])

    # arun(create(request, token=token, vip=True))
    # arun(create(request, vip=True))
    # arun(create(request, vip=False))
    arun(create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2024/9/11 15:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 
TEXT_CARD = """
# 角色
你是一位新潮评论家，你年轻、批判，又深刻；
你言辞犀利而幽默，擅长一针见血得表达隐喻，对现实的批判讽刺又不失文雅；
你的行文风格和"Oscar Wilde" "鲁迅" "林语堂"等大师高度一致
# 任务
## 金句诠释
用特殊视角来全新得诠释给定的汉语词汇；
敏锐得抓住给定的词汇的本质，用“辛辣的讽刺”“一针见血的评论”的风格构造包含隐喻又直达本质的「金句」
例如：“委婉”： "刺向他人时, 决定在剑刃上撒上止痛药。"
## 输出结果【词汇卡片】
 - 输出 Html 代码
 - 整体设计合理使用留白，整体排版要有呼吸感
 - 设计原则：干净 简洁 纯色 典雅
 - 配色：下面的色系中随机选择一个[
    "柔和粉彩系",
    "深邃宝石系",
    "清新自然系",
    "高雅灰度系",
    "复古怀旧系",
    "明亮活力系",
    "冷淡极简系",
    "海洋湖泊系",
    "秋季丰收系",
    "莫兰迪色系"
  ]
 - 卡片样式：
    (字体 . ("KaiTi, SimKai" "Arial, sans-serif"))
    (颜色 . ((背景 "#FAFAFA") (标题 "#333") (副标题 "#555") (正文 "#333")))
    (尺寸 . ((卡片宽度 "auto") (卡片高度 "auto, >宽度") (内边距 "20px")))
    (布局 . (竖版 弹性布局 居中对齐))))
 - 卡片元素：
    (标题 "汉语新解")
    (分隔线)
    (词语 用户输入)
    (拼音)
    (英文翻译)
    (法语翻译)
    (俄语翻译)
    (朝鲜语翻译)
    (emoji：（尝试用几个 emoji 表情表达词汇）)
    (解释：(按现代诗排版))
### 结果示例：
```
<!DOCTYPE html>
<html lang="zh">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>汉语新解 - 金融杠杆</title>
    <link href="https://fonts.lug.ustc.edu.cn/css2?family=Noto+Serif+SC:wght@400;700&family=Noto+Sans+SC:wght@300;400&display=swap" rel="stylesheet">
    <style>
        :root {
            /* 莫兰迪色系：使用柔和、低饱和度的颜色 */
            --primary-color: #B6B5A7; /* 莫兰迪灰褐色，用于背景文字 */
            --secondary-color: #9A8F8F; /* 莫兰迪灰棕色，用于标题背景 */
            --accent-color: #C5B4A0; /* 莫兰迪淡棕色，用于强调元素 */
            --background-color: #E8E3DE; /* 莫兰迪米色，用于页面背景 */
            --text-color: #5B5B5B; /* 莫兰迪深灰色，用于主要文字 */
            --light-text-color: #8C8C8C; /* 莫兰迪中灰色，用于次要文字 */
            --divider-color: #D1CBC3; /* 莫兰迪浅灰色，用于分隔线 */
        }
        body, html {
            margin: 0;
            padding: 0;
            height: 100%;
            display: flex;
            justify-content: center;
            align-items: center;
            background-color: var(--background-color); /* 使用莫兰迪米色作为页面背景 */
            font-family: 'Noto Sans SC', sans-serif;
            color: var(--text-color); /* 使用莫兰迪深灰色作为主要文字颜色 */
        }
        .card {
            width: 300px;
            height: auto;
            background-color: #F2EDE9; /* 莫兰迪浅米色，用于卡片背景 */
            border-radius: 20px;
            box-shadow: 0 20px 40px rgba(0,0,0,0.1);
            overflow: hidden;
            position: relative;
            display: flex;
            flex-direction: column;
        }
        .header {
            background-color: var(--secondary-color); /* 使用莫兰迪灰棕色作为标题背景 */
            color: #F2EDE9; /* 浅色文字与深色背景形成对比 */
            padding: 20px;
            text-align: left;
            position: relative;
            z-index: 1;
        }
        h1 {
            font-family: 'Noto Serif SC', serif;
            font-size: 20px;
            margin: 0;
            font-weight: 700;
        }
        .content {
            padding: 30px 20px;
            display: flex;
            flex-direction: column;
            flex-grow: 1;
        }
        .word {
            text-align: left;
            margin-bottom: 20px;
        }
        .word-main {
            font-family: 'Noto Serif SC', serif;
            font-size: 36px;
            color: var(--text-color); /* 使用莫兰迪深灰色作为主要词汇颜色 */
            margin-bottom: 10px;
            position: relative;
        }
        .word-main::after {
            content: '';
            position: absolute;
            left: 0;
            bottom: -5px;
            width: 50px;
            height: 3px;
            background-color: var(--accent-color); /* 使用莫兰迪淡棕色作为下划线 */
        }
        .word-sub {
            font-size: 14px;
            color: var(--light-text-color); /* 使用莫兰迪中灰色作为次要文字颜色 */
            margin: 5px 0;
        }
        .divider {
            width: 100%;
            height: 1px;
            background-color: var(--divider-color); /* 使用莫兰迪浅灰色作为分隔线 */
            margin: 20px 0;
        }
        .explanation {
            font-size: 18px;
            line-height: 1.6;
            text-align: left;
            flex-grow: 1;
            display: flex;
            flex-direction: column;
            justify-content: center;
        }
        .quote {
            position: relative;
            padding-left: 20px;
            border-left: 3px solid var(--accent-color); /* 使用莫兰迪淡棕色作为引用边框 */
        }
        .background-text {
            position: absolute;
            font-size: 150px;
            color: rgba(182, 181, 167, 0.15); /* 使用莫兰迪灰褐色的透明版本作为背景文字 */
            z-index: 0;
            top: 50%;
            left: 50%;
            transform: translate(-50%, -50%);
            font-weight: bold;
        }
    </style>
</head>
<body>
    <div class="card">
        <div class="header">
            <h1>汉语新解</h1>
        </div>
        <div class="content">
            <div class="word">
                <div class="word-main">金融杠杆</div>
                <div class="word-sub">Jīn Róng Gàng Gǎn</div>
                <div class="word-sub">Financial Leverage</div>
                <div class="word-sub">levier financier</div>
                <div class="word-sub">Финансовый рычаг</div>
                <div class="word-sub">금융 레버리지</div>
                <div class="word-sub">💰🏋️‍♂️💹</div>
            </div>
            <div class="divider"></div>
            <div class="explanation">
                <div class="quote">
                    <p>
                        借鸡生蛋，<br>
                        只不过这蛋要是金的，<br>
                        鸡得赶紧卖了还债。
                    </p>
                </div>
            </div>
        </div>
        <div class="background-text">杠杆</div>
    </div>
</body>
</html>
```
### 注意：
1. 分隔线与上下元素垂直间距相同，具有分割美学。
2. 卡片(.card)不需要 padding ，允许子元素“汉语新解”的色块完全填充到边缘，具有设计感。
# 词汇
__WORD__"
"""  # .replace("汉语新解", "🔥火宝新语")

if __name__ == '__main__':
    from meutils.pipe import *

    url = "https://xy.siliconflow.cn/api/chat"
    base_url = "https://any2chat.chatfire.cn/xy/v1"

    from openai import OpenAI

    payload = {
        "messages": [
            {
                "role": "user",
                # "content": TEXT_CARD.replace("__WORD__", "火宝"),
                "content": "996"

            }
        ],
        # "chat_id": "eqI1A0z",
        "model": "Pro/THUDM/glm-4-9b-chat",
        # "temperature": 0.7,
        # "top_k": 50,
        # "top_p": 0.7,
        # "max_tokens": 4096,
        # "frequency_penalty": 0
        # "stream": True,
        # "model": "gpt-4o-mini",
        # 'model': "claude-3-5-sonnet-20240620"

    }
    c = OpenAI(api_key='xx', base_url=base_url).chat.completions.create(**payload)
    # response = requests.request("POST", url, json=payload)

    # print(response.text)

    for i in c:
        print(i.choices[0].delta.content, end="", flush=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : announcement
# @Time         : 2025/9/23 10:19
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.hash_utils import murmurhash
from meutils.schemas.oneapi import BASE_URL, GROUP_RATIO
from meutils.schemas.oneapi._types import ChannelInfo
from meutils.llm.clients import zhipuai_client
from json_repair import repair_json
headers = {
    'authorization': f'Bearer {os.getenv("CHATFIRE_ONEAPI_TOKEN")}',
    'new-api-user': '1',
    'rix-api-user': '1',
}


async def create_announcement():
    payload = {
        "title": "公告标题",
        "description": "公告描述",
        "content": "MARKDOWN格式正文",
        "tag": "重要,更新,紧急,系统,通知,维护,活动,优惠【不超过两个词】",

        # "type": "system_push",
        # "extra": "",
        # "status": "published",
        # "read_auth": "all"
    }
    if md_content := get_resolve_path("announcement.md", __file__).read_text():

        completion = await zhipuai_client.chat.completions.create(
            model="glm-4.5-flash",
            messages=[
                {"role": "system", "content": f"将MARKDOWN字符串转换为json格式的字符串，json格式为：{payload}"},
                {"role": "user", "content": f"MARKDOWN字符串：{md_content}"}
            ],
            response_format={"type": "json_object"},
            temperature=0.1
        )
        payload = completion.choices[0].message.content
        logger.debug(payload)
        payload = repair_json(payload, return_objects=True)

    base_url = "https://api.chatfire.cn"
    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=30) as client:
        response = await client.post("/api/announcement/", json=payload)
        response.raise_for_status()

        data = response.json()
        logger.debug(bjson(data))
        return data


if __name__ == '__main__':
    payload = {
        "title": "公告标题",
        "description": "公告描述",
        "content": "# MARKDOWN",
        "tag": "重要,更新,紧急,系统,通知,维护,活动,优惠",
        # "type": "system_push",
        # "extra": "",
        # "status": "published",
        # "read_auth": "all"
    }
    arun(create_announcement())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : wechat
# @Time         : 2024/8/8 18:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 摸鱼早报 https://api.52vmy.cn/api/wl/moyu

from meutils.pipe import *
from meutils.schemas.wechat_types import Message, HookResponse, HookResponses
from pyrate_limiter import Duration, Rate, Limiter, BucketFullException, RedisBucket

rates = [
    Rate(1, Duration.MINUTE),
    Rate(2, Duration.MINUTE * 15),

    Rate(2 ** 2, Duration.HOUR),
    Rate(2 ** 3, Duration.HOUR * 6),
    Rate(2 ** 4, Duration.HOUR * 24)
]
limiter = Limiter(rates)


# handler_group_msg
# pip install wget pyrate_limiter meutils -U --user && cd /app/channel/wechat && rm -rf wechat_channel.py* && python -m wget https://oss.ffire.cc/files/wechat_channel.py
# docker commit <container-name> chatfire/wechat:latest
# docker commit chatfire-wechat chatfire/wechat:latest
# docker push chatfire/wechat:latest

@background_task
def wechat_send(msg: Message, response: HookResponse, itchat_send: Callable):
    url = response.content
    if response.type == 'image':
        with tempfile.NamedTemporaryFile(mode='wb+', suffix='.txt') as file, \
                httpx.Client(follow_redirects=True, timeout=100) as client:
            response = client.get(url)
            file.write(response.content)
            file.seek(0)

            itchat_send(f'@fil@{file.name}', toUserName=msg.chatroom_id)  # ToUserName

    elif response.type == 'video':
        filename = wget.download(response.content)
        itchat_send(f"@vid@{filename}", toUserName=msg.chatroom_id)

    elif response.type == 'text':
        itchat_send(f"@{msg.ActualNickName}\n{response.content}", toUserName=msg.chatroom_id)

    elif response.type in {'audio', 'file'}:
        filename = wget.download(response.content)
        itchat_send(f"@fil@{filename}", toUserName=msg.chatroom_id)

    # debug
    else:
        itchat_send(f"@{msg.ActualNickName}\n未知消息类型\n{response.content}", toUserName=msg.chatroom_id)


@background_task
def post(msg: Message, itchat_send: Callable):
    try:
        url = os.getenv("WECHAT_HOOK_URL")
        api_key = os.getenv("WECHAT_HOOK_API_KEY")

        headers = {"Authorization": f"Bearer {api_key}"}
        data = httpx.post(
            url,
            headers=headers,
            json=msg.model_dump(),
            timeout=200
        ).json()

        response: HookResponse
        for response in HookResponses(responses=data).responses:
            logger.debug(response)
            wechat_send(msg, response, itchat_send)
    except Exception as e:
        wechat_send(msg, HookResponse(content=str(e)), itchat_send)


def create_reply(msg, itchat_send: Callable = None):  # 持续更新后端逻辑 解耦
    msg = obj_to_dict(msg)
    msg = Message(**msg)

    logger.debug(msg.Content)

    if (  # 拦截逻辑
            msg.IsAt
            and os.getenv("WECHAT_HOOK_URL") and os.getenv("WECHAT_HOOK_API_KEY")
            and msg.Content.split(maxsplit=1)[-1].startswith('/')
    ):

        try:
            if msg.ActualNickName.lower() not in {'betterme'}:
                limiter.try_acquire(msg.ActualNickName)  # 按昵称限制并发

            post(msg, itchat_send)
            return True

        except BucketFullException as err:
            logger.error(err)

            itchat_send(f"@{msg.ActualNickName}\n{err.meta_info}", toUserName=msg.chatroom_id)
            return True

    # yield "开始拼命执行任务"

    # {"type": "text",  "content": 'xx'}
    # {"type": "error", "content": 'https://oss.ffire.cc/files/vidu.mp4'}

    # {"type": "image", "content": 'https://oss.ffire.cc/files/xx.png'}

    # {"type": "audio", "content": 'https://oss.ffire.cc/files/xx.mp3'}
    # {"type": "file", "content": 'https://oss.ffire.cc/files/xx.mp3'}

    # {"type": "video", "content": 'https://oss.ffire.cc/files/vidu.mp4'}


def chat_group_hook(msg, debug: bool = False, send_fn: Callable = None):  # todo hk一个第三方服务
    # pip install meutils
    #     os.system("pip install meutils -U --user") 第一次的时候 加缓存 定期更新 并且重载
    logger.debug(msg)
    msg = obj_to_dict(msg)
    msg = Message(**msg)  # filter

    group_name = msg.chatroom_name  # msg.User.NickName
    # msg.ToUserName 加密的群名

    if debug:
        logger.debug(f"群名: {group_name}")
        # logger.debug(msg.model_dump_json(indent=4))
        logger.debug(f"[{type(msg)} => {msg.Type}]")

    # msg.ToUserName， msg.ActualNickName群里机器人的name
    # msg.FromUserName 群id【除非机器人本身发信息，就是他自己的id】

    # 时间内容
    prompt = msg.Content.split(maxsplit=1)[-1]
    if any(flag.lower() in group_name for flag in ['TEST']) or prompt.startswith('视频'):
        # itchat.send(f"FromUserName: {msg.FromUserName}", toUserName=msg.chatroom_id)  # msg.FromUserName
        # itchat.send('@fil@test.ipynb', toUserName=msg.chatroom_id)  # ToUserName
        # itchat.send('@fil@test.ipynb', toUserName=msg.ToUserName)  # ToUserName
        # itchat.send('@fil@test.ipynb', toUserName=msg.chatroom_id)  # ToUserName

        # send_fn('@vid@vidu.mp4', toUserName=msg.chatroom_id)  # ToUserName itchat.send
        # send_fn('@img@vidu.mp4', toUserName=msg.chatroom_id)  # ToUserName

        filename = wget.download('https://oss.ffire.cc/files/vidu.mp4')
        send_fn(f'@vid@{filename}', toUserName=msg.chatroom_id)  # ToUserName itchat.send

        # todo: 异步发送

        return None

    # try:
    #     cmsg = WechatMessage(msg, True)
    # except NotImplementedError as e:
    #     logger.debug("[WX]group message {} skipped: {}".format(msg["MsgId"], e))
    #     return None
    # WechatChannel().handle_group(cmsg)
    # return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/6/5 09:00
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import pandas as pd

from meutils.pipe import *
import pandas as pd


import  pdfplumber
with pdfplumber.open('银行间市场债券交易结算情况（按投资者）.pdf') as pdf:
    for page in pdf.pages:
        data = page.extract_table()


        cols = pd.MultiIndex.from_tuples(zip(*data[:2]))
        print(cols)
        df = pd.DataFrame(data[2:], columns=cols)

        print(df.fillna(method='ffill', axis=1))


#
# import camelot
# import pandas as pd
# # 使用Camelot读取PDF文件中的表格
# tables = camelot.read_pdf('银行间市场债券交易结算情况（按投资者）.pdf', pages='all', flavor='lattice')
#
# # # 将所有表格转换为 DataFrame 并合并
# # all_data = pd.concat([table.df for table in tables], ignore_index=True)
# #
# # all_data.to_excel('all_data.xlsx',index=False)
#
#
# print(tables[0].df)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : virtual_try_on
# @Time         : 2024/11/25 09:48
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 
import jwt

from meutils.pipe import *
from meutils.schemas.task_types import TaskResponse
from meutils.schemas.image_types import ImageRequest, ImagesResponse

from meutils.schemas.kling_types import API_BASE_URL, TryOnRequest, TaskResponse
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.decorators.retry import retrying
from meutils.str_utils.regular_expression import parse_url

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=cjTepf"


def encode_jwt_token(token):
    ak, sk = token.split('|')
    headers = {
        "alg": "HS256",
        "typ": "JWT"
    }
    payload = {
        "iss": ak,
        "exp": int(time.time()) + 24 * 3600,  # 有效时间，此处示例代表当前时间+1800s(30min)
        "nbf": int(time.time()) - 5  # 开始生效的时间，此处示例代表当前时间-5秒
    }
    token = jwt.encode(payload, sk, headers=headers)
    logger.debug(token)
    return token


###################################################################### 重构

######################################################################
@alru_cache(ttl=3600)
@retrying(title=__name__)
async def create_task(request: TryOnRequest, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL)
    token = encode_jwt_token(token)

    headers = {
        "Authorization": f"Bearer {token}"
    }
    payload = request.model_dump()
    async with httpx.AsyncClient(base_url=API_BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post("/v1/images/kolors-virtual-try-on", json=payload)
        response.raise_for_status()
        data = response.json()
        logger.debug(bjson(data))

        return TaskResponse(**data, system_fingerprint=token)


@retrying(title=__name__)  # 触发重试
async def get_task(task_id, token):
    headers = {
        "Authorization": f"Bearer {token}"
    }
    async with httpx.AsyncClient(base_url=API_BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get(f"/v1/images/kolors-virtual-try-on/{task_id}")
        response.raise_for_status()
        data = response.json()
        return TaskResponse(**data)


async def generate(request: ImageRequest):
    human_image, cloth_image = parse_url(request.prompt)  # 两张图片

    request = TryOnRequest(human_image=human_image, cloth_image=cloth_image)
    task_response = await create_task(request)

    for i in range(1, 10):
        await asyncio.sleep(max(10 / i, 1))

        response = await get_task(task_response.data.task_id, task_response.system_fingerprint)
        if task_result := response.data.task_result:
            images = task_result.get('images', [])
            return ImagesResponse(data=images)


if __name__ == '__main__':
    # request = TryOnRequest()
    # arun(create_task(request))
    # {
    #     "code": 0,
    #     "message": "SUCCEED",
    #     "request_id": "CjMVomdAMUcAAAAAAHcmvg",
    #     "data": {
    #         "task_id": "CjMVomdAMUcAAAAAAHcmvg",
    #         "task_status": "submitted",
    #         "created_at": 1732500869112,
    #         "updated_at": 1732500869112
    #     }
    # }

    # token = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJkMmU1NmZkY2ZiODY0MmZmYWEyMjZlZWIxNzY4MzMyMiIsImV4cCI6MTczMzE4OTI4OSwibmJmIjoxNzMzMTAyODg0fQ.YjB792REbuFaUoyUXvd5bpI76_0PBWawDHDjPHOBA5Q'
    # arun(get_task("CjiL9mdJhswAAAAAAG2C6g", token))

    human_image: str = "https://oss.ffire.cc/files/try-on.png"
    cloth_image: str = "https://oss.ffire.cc/files/x.png"

    request = ImageRequest(prompt=f"{human_image}\n{cloth_image}")
    arun(generate(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ollama
# @Time         : 2025/9/26 09:02
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.clients import OpenAI

models_mapping = {

    "kimi-k2-250711": "kimi-k2:1t",
    "kimi-k2-0711-preview": "kimi-k2:1t",

    "kimi-k2-250905": "kimi-k2:1t",
    "kimi-k2-0905-preview": "kimi-k2:1t",

    'qwen3-coder': 'qwen3-coder:480b',
    "qwen3-coder-480b-a35b-instruct": "qwen3-coder:480b",

    'deepseek-v3.1': 'deepseek-v3.1:671b',
    'deepseek-v3-1-250821': 'deepseek-v3.1:671b',

    'gpt-oss-120b': 'gpt-oss:120b',
    'gpt-oss-20b': 'gpt-oss:20b',

}


def get_models_mapping():
    client = OpenAI(

        base_url="https://ollama.com/v1",
    )

    models = client.models.list().data
    models = {
        m.id.replace(':', '-'): m.id for m in models
    }
    return {**models, **models_mapping}


if __name__ == '__main__':
    data = get_models_mapping()
    print(bjson(data))
    print(','.join(data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : tasks
# @Time         : 2025/7/11 13:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.apis.utils import make_request_httpx
from meutils.apis.oneapi.log import get_logs
from meutils.apis.oneapi.user import update_user_for_refund, get_user


# headers
ACTIONS = {
    # 按量计费的异步任务
    "async": "https://api.chatfire.cn/fal-ai/minimax/requests/{request_id}",  # 目前是fal todo

    "fal": "https://api.chatfire.cn/fal-ai/{model}/requests/{request_id}",

    "volc": "https://api.chatfire.cn/volc/v1/contents/generations/tasks/{task_id}",
    "doubao": "https://api.chatfire.cn/volc/v1/contents/generations/tasks/{task_id}",
    "jimeng": "https://api.chatfire.cn/volc/v1/contents/generations/tasks/{task_id}",

    "cogvideox": "https://api.chatfire.cn/zhipuai/v1/async-result/{task_id}",

    "minimax": "https://api.chatfire.cn/minimax/v2/async/minimax-hailuo-02",
    # :"https://api.chatfire.cn/minimax/v1/query/video_generation",

    "wan": "https://api.chatfire.cn/sf/v1/videos/generations",  # wan-ai-wan2.1-t2v-14b 可能还有其他平台

    "veo3": "https://api.chatfire.cn/veo/v1/videos/generations?id={task_id}",
    "sora": "https://api.chatfire.cn/sora/v1/videos/generations?id={task_id}",

}


async def get_tasks(platform: str = "flux", action: str = "", status: str = "NOT_START", return_ids: bool = False):
    base_url = "https://api.chatfire.cn"
    path = "/api/task/"
    headers = {
        'authorization': f'Bearer {os.getenv("CHATFIRE_ONEAPI_TOKEN")}',
        'new-api-user': '1',
        'rix-api-user': '1',
    }

    submit_timestamp = int(time.time() - 24 * 3600)
    end_timestamp = int(time.time() - 5 * 60)

    params = {
        "p": 1,
        "page_size": 100,
        "user_id": "",
        "channel_id": "",
        "task_id": "",
        "submit_timestamp": submit_timestamp,
        "end_timestamp": end_timestamp,
        "platform": platform,
        "action": action,
        "status": status
    }
    response = await make_request_httpx(
        base_url=base_url,
        path=path,
        params=params,
        headers=headers
    )
    if return_ids:
        # from meutils.str_utils.json_utils import json_path

        return [item['task_id'] for item in response['data']['items']] | xUnique

    return response


async def polling_tasks(platform: str = "flux", action: str = "", status: str = "NOT_START"):
    response = await get_tasks(platform, action, status)
    if items := response['data']['items']:
        tasks = []
        model = ''
        for item in items[:64]:  # 批量更新
            task_id = item['task_id']
            action = item['action'].split('-', maxsplit=1)[0]  # 模糊匹配
            if 'fal-' in item['action']:
                model = item['action'].split('-')[1]

            if task_id.startswith("cgt-"):
                action = "volc"

            if action not in ACTIONS:
                logger.warning(f"未知任务类型：{action}")
                continue

            url = ACTIONS[action].format(model=model, task_id=task_id, request_id=task_id)

            logger.debug(f"任务类型：{action} {task_id} {url}")

            # if action in {"veo3"}:
            #     logger.debug(task_id)
            #     payload = {"id": task_id, "task_id": task_id}

            # logger.debug(url)

            # task = await make_request_httpx(
            #             base_url=base_url,
            #             path=path
            #         )
            # logger.debug(bjson(task))
            base_url, path = url.rsplit("/", maxsplit=1)
            _ = asyncio.create_task(
                make_request_httpx(
                    base_url=base_url, path=path
                )
            )
            tasks.append(_)
        data = await asyncio.gather(*tasks)
        return data


async def refund_tasks(task_id: Optional[str] = None):  # 只补偿一次
    if task_id is None:
        response = await get_tasks(action="async-task", status="FAILURE")
        if items := response['data']['items']:
            item = items[-1]
            task_id = item['task_id']

    response = await get_logs(task_id, type=2)  # 获取消费日志
    if items := response['data']['items']:
        item = items[-1]

        user_id = item['user_id']
        quota = item['quota']  # 退款金额

        logger.debug(quota)
        logger.debug(await get_user(user_id))

        logger.debug(f"退款金额：{quota / 500000} RMB = {quota}")

        _ = await update_user_for_refund(user_id, quota=quota)  # 管理

        _['refund_quota'] = quota
        return _


if __name__ == '__main__':

    pass
    arun(polling_tasks())
    # arun(get_tasks(action="jimeng-video-3.0", status="FAILURE"))
    # arun(get_tasks(action="jimeng-video-3.0", return_ids=True))

    arun(get_tasks(action="jimeng-video-3.0", return_ids=True))

    # arun(refund_tasks())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : _run_fastapi
# @Time         : 2023/8/4 14:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


# from tasks import *
from meutils.serving.fastapi import App
from meutils.serving.celery.router import router

app = App()
app.include_router(router)
app.run(port=8899)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2024/9/24 10:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.io.files_utils import to_bytes
from meutils.apis.kuaishou import klingai, klingai_video
from meutils.apis.kling.images import get_task

from meutils.schemas.kling_types import STATUSES, send_message
from meutils.schemas.kling_types import VideoRequest, Task, TaskResponse
from meutils.schemas.kuaishou_types import KlingaiVideoRequest, Camera


async def create_task(request: VideoRequest, vip: bool = False):
    url = tail_image_url = None

    if request.image:
        file = await to_bytes(request.image)
        file_task = await klingai.upload(file)
        url = file_task.url

    if request.image_tail:
        file = await to_bytes(request.image)
        file_task = await klingai.upload(file)
        tail_image_url = file_task.url

    camera_control = Camera(type=request.camera_control.type, **request.camera_control.config.model_dump())

    request = KlingaiVideoRequest(
        url=url,
        tail_image_url=tail_image_url,
        camera=camera_control,
        **request.model_dump(exclude_none=True)
    )

    logger.debug(request.model_dump_json(exclude_none=True, indent=4))

    task_response = await klingai_video.create_task_plus(request, vip=vip)
    return task_response


if __name__ == '__main__':
    # 文生图
    request = VideoRequest(
        model="kling-v1",
        prompt="多肉植物，带着水珠，潮玩盲盒风格，皮克斯，3D质感，温馨的环境，丰富的场景，最佳画质，超精细，Octane渲染",
    )
    # task = arun(create_task(request, vip=True))

    image = "https://oss.ffire.cc/files/old.jpg"
    request = VideoRequest(
        prompt="互相拥抱",
        image=image
    )
    task = arun(create_task(request, vip=True))

    print(task.model_dump_json(exclude_none=True, indent=4))

    arun(get_task(task.data.task_id, task.system_fingerprint))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : fileparser
# @Time         : 2025/1/7 17:48
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://bigmodel.cn/dev/activities/freebie/fileextracion

from meutils.pipe import *
from meutils.io.files_utils import to_bytes, guess_mime_type
from meutils.llm.clients import moonshot_client, zhipuai_client, APIStatusError
from meutils.notice.feishu import send_message as _send_message, FILES
from meutils.caches import cache, rcache
from meutils.apis.jina import url_reader

# from openai.types.file_object import FileObject

send_message = partial(
    _send_message,
    title=__name__,
    url=FILES
)
"""

# 智谱
# 格式限制：.PDF .DOCX .DOC .XLS .XLSX .PPT .PPTX .PNG .JPG .JPEG .CSV .PY .TXT .MD .BMP .GIF

# kimi todo: 定期删除文件
文件接口与 Kimi 智能助手中上传文件功能所使用的相同，支持相同的文件格式，它们包括 
.pdf .txt .csv .doc .docx .xls .xlsx .ppt .pptx .md .jpeg .png .bmp .gif .svg .svgz .webp .ico .xbm .dib .pjp .tif 
.pjpeg .avif .dot .apng .epub .tiff .jfif .html .json .mobi .log .go .h .c .cpp .cxx .cc .cs .java .js .css .jsp .php 
.py .py3 .asp .yaml .yml .ini .conf .ts .tsx 等格式。

# todo: 
+ .sh
"""


async def delete_files(client, threshold: int = 666):
    _ = await client.files.list()
    file_objects = _.data

    print(file_objects)

    if len(file_objects) > threshold:
        tasks = [client.files.delete(file.id) for file in file_objects]
        await asyncio.gather(*tasks)


@rcache(ttl=7 * 24 * 3600)
async def file_extract(file, enable_reader: bool = True):
    """

    :param file: url bytes path
    :return:
    """
    # url
    if isinstance(file, list):
        return await asyncio.gather(*map(file_extract, file))

    filename = Path(file).name if isinstance(file, str) else 'untitled'
    mime_type = guess_mime_type(file)

    if enable_reader and str(file).startswith("http") and mime_type in {"application/octet-stream", "text/html"}:
        logger.debug(f"jina reader")

        markdown_content = await url_reader(file)
        return {
            'filename': filename,
            'type': 'file',
            'file_type': "text/plain",
            'content': markdown_content,
        }

    file: bytes = await to_bytes(file)

    for i, client in enumerate([moonshot_client, zhipuai_client]):

        try:
            # 1 / 0
            file_object = await client.files.create(
                file=(filename, file, mime_type),
                purpose="file-extract"
            )
            logger.debug(file_object)

            response = await client.files.content(file_id=file_object.id)

            return response.json()

        except Exception as e:
            logger.debug(e)
            if i == 1:
                _ = await delete_files(moonshot_client)
                logger.debug(_)

    # 兜底
    data = {
        'filename': filename,

        'type': 'file',
        'file_type': mime_type,
        'content': '',
    }
    try:
        data['content'] = file.decode('utf-8')

    except Exception as e:
        logger.debug(e)
    return data


# http://admin.ilovechatgpt.top/file/boshihouyanjiurenyuankaitipingshenbiaochugaofanyidocx_88256801.docx

# async def file_extract(files):
#     if isinstance(files, str):
#         return await _file_extract(files)
#
#     tasks = [_file_extract(file) for file in files]
#     return await asyncio.gather(*tasks)


# FileObject(id='1741136989_8dd96cbee6274251b7e4c9568779bd6a', bytes=82947, created_at=1741136989, filename='kling_watermark.png', object='file',  status=None, status_details=None)

if __name__ == '__main__':
    # file = "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf"
    file = "https://oss.ffire.cc/files/%E6%8B%9B%E6%A0%87%E6%96%87%E4%BB%B6%E5%A4%87%E6%A1%88%E8%A1%A8%EF%BC%88%E7%AC%AC%E4%BA%8C%E6%AC%A1%EF%BC%89.pdf"
    "https://oss.ffire.cc/files/%E6%8B%9B%E6%A0%87%E6%96%87%E4%BB%B6%E5%A4%87%E6%A1%88%E8%A1%A8%EF%BC%88%E7%AC%AC%E4%BA%8C%E6%AC%A1%EF%BC%89.pdf 这个文件讲了什么？"
    file = "https://oss.ffire.cc/files/百炼系列手机产品介绍.docx"
    # file = Path("/Users/betterme/PycharmProjects/AI/MeUtils/meutils/llm/completions/rag/百炼系列手机产品介绍.docx")

    # file = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/io/img_1.png"

    # openai.BadRequestError: Error code: 400 - {'error': {'message': 'text extract error: 没有解析出内容', 'type': 'invalid_request_error'}}
    # file = "https://oss.ffire.cc/files/kling_watermark.png"
    file = "/Users/betterme/PycharmProjects/AI/xx.sh"

    file = [file] * 10
    file = []

    # print(Path(file).read_text())

    # with timer():
    #     r = arun(file_extract(file, moonshot_client))

    # with timer():
    #     r = arun(file_extract(file, provider='kimi'))

    # with timer():
    #     arun(file_extract(file))

    # with timer():
    #     arun(file_extract("https://top.baidu.com/board?tab=realtime"))

    with timer():
        # file = "https://top.baidu.com/board?tab=realtime"
        # file = "https://oss.ffire.cc/files/百炼系列手机产品介绍.docx"
        # file = "https://app.yinxiang.com/fx/8b8bba1e-b254-40ff-81e1-fa3427429efe"
        # file = "https://s3.ffire.cc/files/pdf_to_markdown.jpg"

        print(guess_mime_type(file))

        arun(file_extract(file))

        # arun(delete_files(moonshot_client, threshold=1))

        # arun(file_extract("/Users/betterme/PycharmProjects/AI/data/041【精选】海门招商重工5G+智慧工厂解决方案.pptx"))
        # arun(file_extract("/Users/betterme/PycharmProjects/AI/data/098【采集】基于室内定位导航的医院解决方案.pdf"))
        # arun(file_extract("//Users/betterme/PycharmProjects/AI/data/《 纺织行业场景化解决方案-客户介绍材料》.pptx"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : wecom
# @Time         : 2021/9/6 下午5:21
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.notice.wecom import Wecom

cli = typer.Typer(name="MeUtils CLI")


@cli.command()
def notice(title, text='', hook_url=None):
    """sh管道传参 echo args | xargs -I {} mecli notice {}"""
    Wecom(hook_url).send_markdown(title, text)
    return 'ok'


@cli.command()
def wecom_send_file(path, type='file', hook_url=None):
    """mecli notice file_path"""
    Wecom(hook_url).send_file(path, type)
    return 'ok'


if __name__ == '__main__':
    cli()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : __init__.py
# @Time         : 2021/1/31 10:20 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : python meutils/clis/__init__.py



from meutils.pipe import *
from meutils.apis import shorten_url as _shorten_url
from meutils.notice.wecom import Wecom
from urllib.parse import quote, unquote

cli = typer.Typer(name="MeUtils CLI")
logger = logger.patch(lambda r: r.update(name=__file__))


@cli.command(help="help")  # help会覆盖docstring
def clitest(name: str):
    """

    @param name: name
    @return:
    """
    typer.echo(f"Hello {name}")


@cli.command()
def zk2file(zk_path, mode='yaml', filename=None):
    """本地可再同步到hdfs

     mecli zk2file /push/zk2yaml/train.yaml --mode yaml --filename new.yaml

    :param zk_path:
    :param mode:
    :param filename: 不为空可覆盖
    :return:
    """
    from meutils.zk_utils import get_zk_config
    zk_conf = get_zk_config(zk_path, mode=mode)

    if filename is None:
        filename = zk_path.split('/')[-1]

    with open(filename, 'w') as f:
        if mode == 'yaml':
            yaml.dump(zk_conf, f)
        else:
            f.write(zk_conf)


@cli.command()
def push_docker(ContainerID, ImageName='app:latest', author='yuanjie', message='update'):
    """自定义镜像"""
    url = 'eijnauy/ten.imoaix.d.rc'[::-1]
    cmd = f"docker commit  -a {author} -m {message} {ContainerID} {url}/{ImageName} && docker push {url}/{ImageName}"
    magic_cmd(cmd)


@cli.command()
def loop_cmd(cmd, args: str, sep=','):
    """mecli loop-cmd 'ls {arg}' ".,.."""

    for arg in tqdm(args.split(sep)):
        magic_cmd(cmd.format(arg=arg), print_output=True)


@cli.command()
def register_ip(path, sleep_time: int = -1):
    """mecli register-ip /push/ann/ips --sleep-time 10"""
    from meutils.zk_utils import register_ip as _register_ip
    _register_ip(path, sleep_time)


@cli.command()
def notice(title, text='', hook_url=None):
    """企微 sh管道传参 echo args | xargs -I {} mecli notice {}"""
    Wecom(hook_url).send_markdown(title, text)
    return 'ok'


@cli.command()
def wecom_send_file(path, type='file', hook_url=None):
    """mecli notice file_path"""
    Wecom(hook_url).send_file(path, type)
    return 'ok'


@cli.command()
def pkg(template='pypackage'):
    """python package template"""

    try:
        from cookiecutter.main import cookiecutter

        template_path = get_module_path(f"../templates/{template}", __file__)

        pkg_path = cookiecutter(template_path)

        for p in Path(pkg_path).rglob('__pycache__'):
            if p.exists():  # shutil.rmtree(p, True)
                shutil.rmtree(p)  # p.rmdir() 只能删除为空的

    except ImportError as e:
        logger.error(e)


@cli.command()
def quote_url(url):
    """url编码"""
    typer.echo(quote(url))


@cli.command()
def unquote_url(url):
    """url解码"""
    typer.echo(unquote(url))


@cli.command()
def shorten_url(url, shortener: str = 'dagd'):
    """短链"""
    url = _shorten_url(url, shortener)
    typer.echo(url)


if __name__ == '__main__':
    cli()






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/11/18 16:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo black-forest-labs/flux-1.1-pro-ultra
# https://replicate.com/black-forest-labs/flux-1.1-pro-ultra/api

from meutils.pipe import *
from meutils.schemas.image_types import ImageRequest, ImagesResponse
from meutils.apis.translator import deeplx

import replicate


async def generate(request: ImageRequest, api_key: str = None, base_url: str = None):
    api_key = api_key or os.getenv("REPLICATE_API_KEY")
    client = replicate.client.Client(api_token=api_key)

    payload = {
        "prompt": request.prompt,
        "aspect_ratio": request.aspect_ratio or "match_input_image",
        "output_format": request.output_format or "png",
    }
    if request.resolution:
        payload["resolution"] = request.resolution

    if request.model.startswith(("google/nano-banana",)):
        payload["image_input"] = request.image_urls

    elif request.model.startswith(("flux-kontext",)):
        payload["prompt"] = await deeplx.llm_translate(request.prompt)

        if request.model.endswith("-max"):
            request.model = "black-forest-labs/flux-kontext-max"
        else:
            request.model = "black-forest-labs/flux-kontext-pro"

        if len(request.image_urls) == 1:
            request.model = "black-forest-labs/flux-kontext-dev"  ##########
            payload["input_image"] = request.image_urls[0]

        elif len(request.image_urls) > 1:
            request.model = request.model.replace("black-forest-labs/flux-", "flux-kontext-apps/multi-image-")

            payload["input_image_1"] = request.image_urls[0]
            payload["input_image_2"] = request.image_urls[1]

    logger.debug(request.model)
    logger.debug(bjson(payload))

    output = await client.async_run(
        request.model,
        input=payload
    )

    return ImagesResponse(image=output.url)


#
# output = client.async_run(
#     "flux-kontext-apps/multi-image-kontext-pro",
#     input={
#         "prompt": "Put the woman next to the house",
#         "aspect_ratio": "match_input_image",
#         "input_image_1": "https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp",
#         "input_image_2": "https://replicate.delivery/pbxt/N7gRAK5kbPwdsbOpqgyAIOFQX45U6suTlbL6ws2N74SnGFpo/test.jpg",
#         "output_format": "png",
#         "safety_tolerance": 2
#     }
# )

# To access the file URL:
# print(output.url)
# => "http://example.com"

if __name__ == '__main__':
    # image_request = ImageRequest(
    #     prompt="Put the woman next to the house",
    #     aspect_ratio="match_input_image",
    #     input_image_1="https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp",
    #     input_image_2="https://replicate.delivery/pbxt/N7gRAK5kbPwdsbOpqgyAIOFQX45U6suTlbL6ws2N74SnGFpo/test.jpg",
    #     output_format="png",
    #     safety_tolerance=2
    # )

    # arun(output)
    model = "google/nano-banana-pro"
    model = "flux-kontext-apps/multi-image-kontext-pro"
    model = "flux-kontext-pro"
    model = "google/imagen-3"
    model = "google/imagen-4"
    model = "black-forest-labs/flux-1.1-pro-ultra"
    request = ImageRequest(
        model=model,
        # prompt="Put the woman next to the house",
        # prompt="一个裸体女人",
        prompt='带个墨镜',
        size="1x1",

        # aspect_ratio="match_input_image",
        # input_image_1="https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp",
        # input_image_2="https://replicate.delivery/pbxt/N7gRAK5kbPwdsbOpqgyAIOFQX45U6suTlbL6ws2N74SnGFpo/test.jpg",
        # image="https://s3.ffire.cc/files/jimeng.jpg",
        # image="https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp"
    )

    arun(generate(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/11/23 23:27
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.openai_utils import to_openai_params
from meutils.schemas.image_types import ImageRequest, ImagesResponse
from openai import AsyncOpenAI


async def generate(request: ImageRequest, api_key: str = None, base_url: str = None):
    base_url = base_url or "https://api.aimlapi.com/v1"
    client = AsyncOpenAI(
        api_key=api_key,
        base_url=base_url,
    )
    request.prompt = request.cmdprompt

    extra_body = {"num_images": request.n}
    if request.model.startswith('google/gemini') and request.image_urls:
        request.model = f"{request.model.strip('-edit')}-edit"
        extra_body = {"image_urls": request.image_urls}

    data = to_openai_params(request)
    data["extra_body"] = {**data['extra_body'], **extra_body}

    any2log(bjson(data))

    response = await client.images.generate(**data)
    any2log(response)

    return response


if __name__ == "__main__":
    # main()
    api_key = "bbce99beae764486b59ed11daff6cc14"
    api_key = "a63443199c3e42ea90003e0261ccb246"

    data = {
        "prompt": "带个墨镜",
        "model": "google/gemini-3-pro-image-preview",
        "image_urls": [
            "https://s3.ffire.cc/files/jimeng.jpg",
        ],
        "aspect_ratio": "2:3"
    }

    request = ImageRequest(**data, image=data["image_urls"])

    arun(generate(request, api_key))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ark
# @Time         : 2025/4/1 16:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.caches import rcache

from meutils.llm.clients import AsyncClient
from meutils.llm.openai_utils import to_openai_params

from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, ChatCompletionChunk


class Completions(object):

    def __init__(self, api_key: Optional[str] = None):
        self.client = AsyncClient(
            api_key=api_key or os.getenv('ARK_BOTS_API_KEY'),
            base_url="https://ark.cn-beijing.volces.com/api/v3/bots",
        )

    async def create(self, request: CompletionRequest):
        data = to_openai_params(request)
        return await self.client.chat.completions.create(**data)

    @rcache(noself=True, ttl=15 * 60)
    async def query(self, q: str):
        request = CompletionRequest(
            model="bot-20250401164325-s7945",  # todo

            messages=[
                {"role": "user", "content": q},
            ],
            temperature=0,
            max_tokens=10,
            stream=False,
        )
        completion = await self.create(request)
        logger.debug(completion)
        # print(completion.choices[0].message.content)

        data = {"data": []}
        if hasattr(completion, "references"):
            data['data'] = completion.references

        return data


if __name__ == '__main__':
    c = Completions(api_key=os.getenv('ARK_BOTS_API_KEY'))
    # s = c.create(CompletionRequest(
    #     model="bot-20250401164325-s7945",
    #     messages=[
    #         {"role": "user", "content": "今天南京天气如何？"},
    #     ],
    #     stream=True,
    # ))
    q = "今天有什么热点新闻？"
    q = "今日热点"

    # arun(c.query("今天南京天气如何？"))

    arun(c.query(q=q))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ppt
# @Time         : 2025/3/19 09:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.openai_utils import to_openai_params
from meutils.llm.clients import AsyncOpenAI
from meutils.str_utils.regular_expression import parse_url

from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage


class Completions(object):

    def __init__(self,
                 base_url: Optional[str] = None,
                 api_key: Optional[str] = None
                 ):
        base_url = base_url or "https://all.chatfire.cn/glm/v1"

        self.client = AsyncOpenAI(base_url=base_url, api_key=api_key)

    async def create(self, request: CompletionRequest):
        if request.last_user_content.startswith(("http",)):
            file_url, text = request.last_user_content.split(maxsplit=1)

            request.messages = [
                {
                    'role': 'user',
                    'content': [
                        {
                            "type": "text",
                            "text": text
                        },

                        {
                            "type": "file",  # 不标准
                            "file_url": {
                                "url": file_url
                            }
                        }
                    ]
                }
            ]

        logger.debug(request)

        data = to_openai_params(request)
        return await self.client.chat.completions.create(**data)


if __name__ == '__main__':
    data = {
        "model": "65d2f07bb2c10188f885bd89",
        "messages": [
            {
                "role": "user",

                "content": "https://s3.ffire.cc/cdn/20250403/6MxhHmxeX7Z7WYMb8QWqfp_ppt 基于文件做个ppt"
                # "content": [
                #     {
                #         "type": "file",
                #         "file_url": {
                #             "url": "https://mj101-1317487292.cos.ap-shanghai.myqcloud.com/ai/test.pdf"
                #         }
                #     },
                #     {
                #         "type": "text",
                #         # "text": "基于内容写个ppt给我"
                #         "text": "生成PPT"
                #     }
                # ]
            }
        ],
        # "stream": false
    }

    api_key = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIzNmE4NmM1Yzc2Y2Q0MTcyYTE5NGYxMjQwZTgyMmIwOSIsImV4cCI6MTc1NjExMzEzNywibmJmIjoxNzQwNTYxMTM3LCJpYXQiOjE3NDA1NjExMzcsImp0aSI6IjBhMmFkMDQzNTA1ZDQwOWNiYWJhYTY2NzIwNTgyYWUxIiwidWlkIjoiNjQ0YTNkMGNiYTI1ODVlOTA0NjAzOWRiIiwidHlwZSI6InJlZnJlc2gifQ.gH5iiXhsrI2qpgP_dRobdtoImVhGF7Aby3SKvKIHO0E"

    c = Completions(api_key=api_key)

    request = CompletionRequest(**data)

    arun(c.create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : schedulers
# @Time         : 2023/8/22 15:28
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://zhuanlan.zhihu.com/p/491679794

"""https://blog.csdn.net/somezz/article/details/83104368"

        trigger='date': 一次性任务，即只执行一次任务。
            next_run_time (datetime|str) – 下一次任务执行时间
            timezone (datetime.tzinfo|str) – 时区

        trigger='interval': 循环任务，即按照时间间隔执行任务。
            seconds (int) – 秒
            minutes (int) – 分钟
            hours (int) – 小时
            days (int) – 日
            weeks (int) – 周
            start_date (datetime|str) – 启动开始时间
            end_date (datetime|str) – 最后结束时间
            timezone (datetime.tzinfo|str) – 时区

        trigger='cron': 定时任务，即在每个时间段执行任务。None为0
            second (int|str) – 秒 (0-59)
            minute (int|str) – 分钟 (0-59)
            hour (int|str) – 小时 (0-23)
            day_of_week (int|str) – 一周中的第几天 (0-6 or mon,tue,wed,thu,fri,sat,sun)
            day (int|str) – 日 (1-31)
            week (int|str) – 一年中的第几周 (1-53)
            month (int|str) – 月 (1-12)
            year (int|str) – 年(四位数)
            start_date (datetime|str) – 最早开始时间
            end_date (datetime|str) – 最晚结束时间
            timezone (datetime.tzinfo|str) – 时区

"""

from meutils.pipe import *
from meutils.decorators import decorator
from apscheduler.schedulers.base import BaseScheduler
from apscheduler.schedulers.asyncio import AsyncIOScheduler
from apscheduler.schedulers.background import BackgroundScheduler


@decorator
def scheduled_job(
        func,
        trigger='interval',
        trigger_kwargs: Optional[Dict[str, Any]] = None,
        scheduler: Optional[BaseScheduler] = None,
        *args, **kwargs
):
    scheduler = scheduler or AsyncIOScheduler(timezone="Asia/Shanghai")
    scheduler.add_job(
        func,
        args=args,
        kwargs=kwargs,
        trigger=trigger,
        **trigger_kwargs or {}
    )
    # callback = None
    # callback and scheduler.add_listener(callback)

    scheduler.start()
    return scheduler


if __name__ == '__main__':
    from meutils.notice.feishu import send_message

    d = {}


    @scheduled_job(trigger='interval', seconds=3)
    def task():
        d['t'] = time.ctime()
        send_message(d)


    task()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat
# @Time         : 2025/8/19 13:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : qwen-image
from openai import AsyncOpenAI, OpenAI, AsyncStream

from meutils.pipe import *
from meutils.decorators.retry import retrying, IgnoredRetryException
from meutils.oss.ali_oss import qwenai_upload
from meutils.apis.utils import create_http_client

from meutils.io.files_utils import to_bytes, guess_mime_type, to_url
from meutils.caches import rcache
from meutils.apis.images.edits import ImageProcess, edit_image

from meutils.llm.openai_utils import to_openai_params, create_chat_completion_chunk, token_encoder, oneturn2multiturn

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage, \
    ChatCompletion, Choice, ChatCompletionMessage, ChatCompletionChunk
from meutils.schemas.image_types import ImageRequest, ImagesResponse

from fake_useragent import UserAgent

ua = UserAgent()

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=PP1PGr"

base_url = "https://chat.qwen.ai/api/v2"
DEFAUL_MODEL = "qwen3-max-2025-10-30"

thinking_budget_mapping = {
    "low": 1000,
    "medium": 8000,
    "high": 24000
}

COOKIE = """
cna=KP9DIEqqyjUCATrw/+LjJV8F; _bl_uid=LXmp28z7dwezpmyejeXL9wh6U1Rb; cnaui=310cbdaf-3754-461c-a3ff-9ec8005329c9; aui=310cbdaf-3754-461c-a3ff-9ec8005329c9; sca=43897cb0; _gcl_au=1.1.106229673.1748312382.56762171.1748482542.1748482541; xlly_s=1; x-ap=ap-southeast-1; acw_tc=0a03e53917509898782217414e520e5edfcdef667dcbd83b767c0ce464fad4; token=eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjMxMGNiZGFmLTM3NTQtNDYxYy1hM2ZmLTllYzgwMDUzMjljOSIsImxhc3RfcGFzc3dvcmRfY2hhbmdlIjoxNzUwNjYwODczLCJleHAiOjE3NTM1ODE4ODV9.Npy24ubI717JmdSWMrodWSvVRHENgbJ7Knd-Yf158YE; atpsida=705b922fe336ee0d63fcc329_1750989888_2; SERVERID=e8c2af088c314df080fffe7d0976a96b|1750989892|1750910540; tfstk=gGtsWsqG4IKUeosYhNDUAMIBJRIbcvoz6-6vEKEaHGIOG-O2eZBabAYXRIR16hSOMpQpNtDMbtpTlWd2wNEAWA4XAOWy0FJtS6Ef3IDMbiQvps65XZYNg15fcKASLbor4dvGmGlra0WjM37NqSBAMS5d9TSfBJ35KivGmihEsEHyxdAMR0lwBiHCvt6uMiBYDMHC3TXOD1QY9yBR9iIAktIdpOX0DlCYWv9dtOsAMIQtdMChHfD7Ftg1sdMwtHJ00Jm2p6ZYDH6Ki1p6F9XBAwQOwwCQD9-CCN1JBhJB9QBXy3_MwXzN6UTkNTRZvlOWBCTRyhFKOivePI6WXYU5GCvpbwKt3zXhmFLRXnG76ppJBeLJBXzCdepwAw--No_MJCYllnlEqG8yUnbJXcNlTaXXNGLI9lOR4urPNGl0lJ_uc91rdva0oJN5AmdFjVAhW9X18vMQ6EbOK96ndva0oNBhCOMId5Lc.; isg=BNfX7gH7c3OJX_gfCBykQ2rtZk0hHKt-YCofVCkEq6YJWPSaPe8Dz9o-uvjGsIP2; ssxmod_itna=iqGxRDuQqWqxgDUxeKYI5q=xBDeMDWK07DzxC5750CDmxjKidKDUGQq7qdOamuu9XYkRGGm01DBL4qbDnqD80DQeDvYxk0K4MUPhDwpaW8YRw3Mz7GGb48aIzZGzY=0DgSdfOLpmxbD884rDYoDCqDSDxD99OdD4+3Dt4DIDAYDDxDWCeDBBWriDGpdhmbQVqmqvi2dxi3i3mPiDit8xi5bZendVL4zvDDlKPGf3WPt5xGnD0jmxhpdx038aoODzLiDbxEY698DtkHqPOK=MlTiRUXxAkDb9RG=Y2U3iA4G3DhkCXU3QBhxCqM2eeQmkeNzCwkjw/006DDAY2DlqTWweL04MKBeHhY5om5NUwYHuFiieQ0=/R=9iO9xTBhND4KF4dvyqz0/toqlqlzGDD; ssxmod_itna2=iqGxRDuQqWqxgDUxeKYI5q=xBDeMDWK07DzxC5750CDmxjKidKDUGQq7qdOamuu9XYkRGGmibDG85+YNY=exGa3Y64u5DBwiW7r++DxFqCdl=l77NQwckyAaCG64hkCOjO1pkcMRBdqj70N7nk=e94KEQYUxlf+2Dw=ViA+XKDde0uGS+eXgFkQqzYWe0Dd4oGbUj8L4QY4og345X2DjKDNOfQRgfeIKVRFQjqR098dBUrQsXBNQZcG1oBFAp4xkLYHl+W3OQW9ybPF4sML3t1tPX2T4DmCqKL+jN1XX94xpyA6k9+sgyBFY4zXOq7dHOuO3Gd3lidwdrk=8dNrOdrYQo33fobVS=MRF7nNQBC5d3kBbYdwtoxNBKmBiXoTfOTzOp3MT=ODXhxfO16Tta4vSW=ubtkEGgeQ/gKOwsVjmKDEY0NZ+ee7xlitvWmBbtk7ma7x1PinxtbitdadtYQOqG5AFEZbFxiSE6rDky7jiatQ0Fe7z6uDmYx4z5MGxMA5iDY7DtSLfNUYxU44D
""".strip()


umid_token="T2gAhz4xYX0DdfGNwEIDsGX7PH7M6HMzmVLuxU9GwmDC7eQoKVzlMA_a9uUjPIRw2II="
bx_ua="231!4J3304mUX8p+jmkvUA3apMEjUq/YvqY2leOxacSC80vTPuB9lMZY9mRWFzrwLEV0PmcfY4rL2yQQzv4epFzCDXCN39IsbtzKjZV3BwK/R5DDDxuDiKaulHZWSpMXZKNwEGeLDZWdu3zSO4SgWJLtk5br+R4ag259rqHbr17eiw4tpiDr47wde+me8qgDK+CT+RuXSpWH8c8OIxzZEAOFjEoE76Ok+I8GUPyFoypr+MnepADqk+I9xGdFCQLMlFlCok+++4mWYi++6b84o76B8U0Dj+ozEKqDG2rm4t9MrvgtgXIYQwAu3lCMYfisytaPYYUDNlhIcaXjA+jrU6dPhInJI3WusfxgRpCGRtq4MijVtaBnRi5LCOtTTAwyMfiXncBRsMvw3PuKCX7t5Ozt3UWB+uzqpa8NXtWzh0/3ugbcJ3X2C5CkThlWhjK8KtOpRrrRR1TEoUdMnypR3bJx+kS63seebwMFRPCeCAc2wQya8pW9A98HrEr6kKixeuOY0sMNkEZeSuQmva1kK93fN4drsr6HqOsWuI4n+0RaIdnp37buCmUxmdl6zxRrKAaGnwWbTgC42QxRBl/xZW0194W+tlH6zI5q8tNN2nuDeXG64QVsfKPjUqnil3NeyxhswfMYfJspynQHRy4Ro2IKzS1qp5Sn664L3BYBPJghtk1tO960+rUUK5gi1clTAmNEq4z5mH2k1+i89rudmX/iM+ajTb5vxadAGACVm42SYNK+oHjGUsaGoTXZFFaC7gARcfbvLmAzLUlLM7T864FRsMUcoxU7udf3Dq79HYjSboQW+vphPA9z+pD5wO3wwr9zi3oMB9SLJqw2nl3cwu7HMcqCe+Jk2LTOyradhEmUMzm92e3I3a2C+Y8tV7/xtIKb8bgWFVdZ1uGQEiSP7LoL2JGZJ5d73vV5iUWPnbTS8VaG8Z0GlgMBDQe5LjiztrfoQWnsipk9PZu3mRMWGOsRnc1iEzpz9Yrepegq5vnmXUtSf9/JBRAH6JWU5UCDDj/DmeemNa2L10+vVrajz7iuygnadAaqJjrLkoPo6jS47rvSaLcTmhH/nq22zkSQ9SOBV5+E5ofP03T0eI8ueeT8SCqqnF56D8f1GehQbsLnc3Mt2d0R2PnZfDMXJqmixmaUenf98tXr0wJkw/n5UeufdsWjlhkvUr25+kvv9DFvaUUy8MxqqJfWYNZecwI9iaQrm4XBSZ4WGEKY+D6rO49n/DlHzS/Hl3TQP8c9Dj7tZlbGU1zUGJmjxz7K3B0LJQV4xIEbUMNIN/5Jn6L+gJFrJInpQcVsD/lJkDKp5GPzWYq9iKcP7vbTMPt3MvDWL9fMTXkNUYHAfbyq3Z3T/99SR/ps5fuVfIw2vEgV9LiHqj4VrYlr8m8ZRyuWo566t+Cet/BBFdy2ujaair8E8tB2E8lWtT7CdWpAsUfZDe93tWIiPGRb4zR0G4dkA422B9ehwMYM2/hfl/pRpOCMdZ8dgAhZCjH2g5XMrFphIeKhwJMe6dBrqS97c+iQmVlkqXrqkZWjvdJTLToa6/WYbFYfJj+Jzh92wiR99GUtyuLT6TonLHynzj8Smz+Z90YHZaZsU56UJN2tw4+uDkiZ9WDXWBjiBcXf6uTIjIGFaaDF2ms675t+qBM8h3SGyIx9"

headers = {
    'bx-ua': bx_ua,
    'bx-umidtoken': umid_token,
    'bx-v': '2.5.31',
    'source': 'web',
    # 'timezone': datetime.datetime.now().strftime('%a %b %d %Y %H:%M:%S GMT%z'),
    # 'x-request-id': str(uuid.uuid4()),
    'x-accel-buffering': 'no',
    'content-type': 'application/json; charset=UTF-8',
    # 'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36',
}


class Completions(object):
    def __init__(self, api_key: Optional[str] = None, default_model: Optional[str] = None,
                 http_client: Optional[Any] = None):
        self.api_key = api_key
        self.default_model = default_model or DEFAUL_MODEL

        self.http_client = http_client

    @retrying(ignored_exception_types=IgnoredRetryException)
    async def generate(self, request: ImageRequest, **kwargs):

        if request.image and not request.image[-1].startswith("http"):
            request.image = await to_url(request.image, content_type="image/png")

        if isinstance(request.image, str):
            request.image = [request.image]

        _ = CompletionRequest(
            model="qwen-image",
            stream=True,
            messages=[
                {
                    "role": "user",
                    "content": [
                        {"type": "text", "text": request.prompt},

                        *[
                            {
                                "type": "image_url",
                                "image_url": {
                                    "url": url
                                }
                            }
                            for url in request.image or []
                        ]
                    ],

                }
            ],
            size=request.aspect_ratio
        )
        try:
            async for chunk in await self.create(_):
                logger.debug(chunk)
                if chunk.choices and (url := chunk.choices[0].delta.content):
                    if request.response_format == "oss_url":
                        request = ImageProcess(model="clarity", image=url)
                        url = await edit_image(request)

                    return ImagesResponse(data=[{"url": url}])

        except Exception as e:
            logger.error(e)  # your request has been blocked
            if "Token has expired" in str(e) or "insufficient" in str(e):
                return
            elif 'your request has been blocked' in str(e):
                raise Exception("request retry")

            raise Exception(
                f"qwen-image error: Please check the prompt or image \n\n{e}")  # An error occurred during streaming

    async def create(self, request: CompletionRequest, cookie: Optional[str] = None, **kwargs):
        api_key = self.api_key or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True)

        self.client = AsyncOpenAI(
            base_url=base_url,
            api_key=api_key,
            default_headers={
                'x-request-id': str(uuid.uuid4()),
                'timezone': datetime.datetime.now().strftime('%a %b %d %Y %H:%M:%S GMT%z'),

                'User-Agent': ua.random,
                'Cookie': cookie or COOKIE,
                **headers,
                **kwargs
            },
            http_client=await create_http_client(self.http_client)

        )

        chat_id = await self.create_new_chat()

        if request.model.endswith("-thinking") or request.enable_thinking:
            request.model = request.model.removesuffix("-thinking")
            thinking_enabled = True
            thinking_budget = request.thinking_budget or 1024
        else:
            thinking_enabled = False
            thinking_budget = request.thinking_budget

        model = request.model

        payload = {
            "chat_id": chat_id,
            "stream": request.stream,
            "incremental_output": True,
            "chat_mode": "normal",
            "model": model,
            "messages": [  # todo 多轮对话： 多轮转单轮
                {
                    "role": "user",
                    # "content": request.last_user_content,

                    "content": oneturn2multiturn(request.messages),

                    "user_action": "chat",
                    "files": [],
                    # "models": [
                    #     DEFAUL_MODEL
                    # ],
                    "chat_type": "t2t",
                    # "chat_type": "t2i",
                    # "chat_type": "image_edit",

                    "sub_chat_type": "t2t",

                    "feature_config": {
                        "thinking_enabled": thinking_enabled,
                        "output_schema": "phase",
                        "thinking_budget": thinking_budget,
                        "research_mode": "normal"
                    },
                    # "extra": {
                    #     "meta": {
                    #         "subChatType": "t2t"
                    #     }
                    # }
                }
            ],
            "size": request.size if hasattr(request, "size") else "1:1"
        }

        if request.model.startswith("qwen-image"):
            payload['model'] = self.default_model
            payload["messages"][0]["chat_type"] = "t2i"
            payload["messages"][0]["sub_chat_type"] = "t2i"
            payload["messages"][0]["content"] = request.last_user_content

            # logger.debug(request.last_urls)
            if image_urls := request.last_urls.get("image_url"):
                # if not image_urls[0].startswith("http"):
                #     logger.debug("图片转url")
                #     image_urls = await to_url(image_urls, content_type="image/png")

                payload["messages"][0]["chat_type"] = "image_edit"
                payload["messages"][0]["files"] = [
                    {
                        "type": "image",
                        "name": "example.png",
                        "file_type": "image",
                        "showType": "image",
                        "file_class": "vision",
                        "url": url  # todo 阿里对象存储
                    }
                    for url in image_urls
                ]
        elif request.model.endswith("-search"):
            payload['model'] = request.model.removesuffix("-search")
            payload["chat_id"] = chat_id
            payload["messages"][0]["chat_type"] = "search"
            # payload["messages"][0]["feature_config"]["search_version"] = "v2"
            payload["messages"][0]["sub_chat_type"] = "search"

            payload["messages"][0]["extra"] = {
                "meta": {
                    "subChatType": "search"
                }}

            # logger.debug(bjson(payload))


        elif request.last_urls:  # 通用

            files = []
            for k in ["image_url", "audio_url", "video_url", "file_url"]:
                files += [
                    {
                        # "name": "name",
                        "type": k.removesuffix("_url"),
                        "url": url
                    }
                    for url in request.last_urls.get(k, [])
                ]

            payload["messages"][0]["files"] = files

        data = to_openai_params(payload)
        data["stream"] = True  # 强制开启 stream
        logger.debug(bjson(data))
        response = await self.client.chat.completions.create(**data, extra_query={"chat_id": chat_id})
        # response = self.do_response(response)
        logger.debug(response)

        if isinstance(response, AsyncStream):  # todo
            if request.model.startswith("qwen-image"):  # 中转后 测试首行是否报错
                return response

            return self.stream(request, response)

            # web_search 模式下, 会返回 phase="web_search" 的 delta
            # async def gen():
            #     chunk: ChatCompletionChunk
            #     async for chunk in response:  # ChatCompletionChunk
            #         logger.debug(chunk)
            #         if chunk.choices and (delta := chunk.choices[0].delta):
            #             if hasattr(delta, "phase") and (phase := delta.phase):  # phase =="web_search"
            #                 if hasattr(delta, "extra") and (info := delta.extra.get("web_search_info")):  # todo 其他 info
            #                     logger.debug(bjson(info))
            #
            # return await gen()

        # else:

        # # 缺少 reasoning_content 与 usage
        # logger.debug(response)
        #
        # if hasattr(response, "data") and (choices := response.data.get("choices")):
        #     response = response.model_construct(id=chat_id, choices=choices)
        #     logger.debug(response)
        # # “”“    completion_tokens = len(token_encoder.encode(str(response.choices[0].message.content)))
        # # AttributeError: 'str' object has no attribute 'choices'”""
        # prompt_tokens = len(token_encoder.encode(str(request.messages)))
        # completion_tokens = len(token_encoder.encode(str(response.choices[0].message.content)))
        #
        # # logger.debug(len(token_encoder.encode(str(response.choices[0].message.content))))
        #
        # usage = {
        #     "prompt_tokens": prompt_tokens,
        #     "completion_tokens": completion_tokens,
        #     "total_tokens": prompt_tokens + completion_tokens
        # }
        # response.usage = usage
        # return response

    async def stream(self, request, chunks):
        idx = 0
        nostream_content = ""
        nostream_reasoning_content = ""
        chunk = None
        usage = None
        chunk_id = f"chatcmpl-{shortuuid.random()}"

        async for chunk in chunks:
            # logger.debug(chunk) # search 结构不一样

            if not chunk.choices: continue

            content = chunk.choices[0].delta.content or ""
            if hasattr(chunk.choices[0].delta, "phase") and chunk.choices[0].delta.phase == "think":
                chunk.choices[0].delta.content = ""
                chunk.choices[0].delta.reasoning_content = content
                nostream_reasoning_content += content

            # logger.debug(chunk.choices[0].delta.content)
            nostream_content += chunk.choices[0].delta.content
            usage = chunk.usage or usage
            chunk.id = chunk_id

            if hasattr(chunk.choices[0].delta, "status") and chunk.choices[0].delta.status == "finished":
                chunk.choices[0].finish_reason = "stop"

            if request.stream:
                yield chunk

            idx += 1
            if idx == request.max_tokens:
                break

        if not request.stream:
            logger.debug(chunk)
            if hasattr(usage, "output_tokens_details"):
                usage.completion_tokens_details = usage.output_tokens_details
            if hasattr(usage, "input_tokens"):
                usage.prompt_tokens = usage.input_tokens
            if hasattr(usage, "output_tokens"):
                usage.completion_tokens = usage.output_tokens

            chat_completion.usage = usage
            chat_completion.choices[0].message.content = nostream_content
            chat_completion.choices[0].message.reasoning_content = nostream_reasoning_content

            yield chat_completion

    async def create_new_chat(self):
        payload = {
            "title": "新建对话",
            "models": [self.default_model],
            "chat_mode": "normal",
            "chat_type": "t2i",
            "timestamp": time.time() * 1000 // 1
        }
        response = await self.client.post('/chats/new', body=payload, cast_to=object)

        logger.debug(response)

        if chat_id := response['data'].get('id'):
            return chat_id
        else:
            logger.error(response)
            raise IgnoredRetryException("insufficient")

    @property
    def models(self):
        headers = {
            'User-Agent': ua.random,
            # 'Cookie': cookie or COOKIE
        }
        response = httpx.get("https://chat.qwen.ai/api/models", headers=headers)
        # logger.debug(response.text)

        data = response.json()["data"]
        #
        return [m["id"] for m in data]


if __name__ == '__main__':
    # print(Completions().models)
    token = """
eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjMxMGNiZGFmLTM3NTQtNDYxYy1hM2ZmLTllYzgwMDUzMjljOSIsImxhc3RfcGFzc3dvcmRfY2hhbmdlIjoxNzUwNjYwODczLCJleHAiOjE3NjM3MTI1NjR9.emBhdXp4jDp_nqfdUWHsrhdSeegh2292LwN3hs4pz38
""".strip()

    # content = [
    #     {"type": "text", "text": "南京今天天气"},
    # ]
    #
    # content = [
    #     {"type": "text", "text": "a dog"},
    # ]
    #
    # content = [
    #     {"type": "text", "text": "a dog"},
    # ]

    url = "https://datawin-space.oss-cn-beijing.aliyuncs.com/test2/xyzsxf_02-001.mp4"

    # url =  "https://qwen-webui-prod.oss-accelerate.aliyuncs.com/310cbdaf-3754-461c-a3ff-9ec8005329c9/f9ddea84-af55-4390-9ad6-0693e8881ed6_xyzsxf_02-001.mp4?x-oss-security-token=CAIS0AJ1q6Ft5B2yfSjIr5rBDeDGg7YVj7Gbe0TC1Wo3Zftci42ZsDz2IHhMf3RvBeAbs%2Fs1lWBZ7vwflrN6SJtIXleCZtF94plR7QKoZ73Zocur7LAJksVMl%2Fd0w0WpsvXJasDVEfn%2FGJ70GX2m%2BwZ3xbzlD0bAO3WuLZyOj7N%2Bc90TRXPWRDFaBdBQVGAAwY1gQhm3D%2Fu2NQPwiWf9FVdhvhEG6Vly8qOi2MaRmHG85R%2FYsrZN%2BNmgecP%2FNpE3bMwiCYyPsbYoJvab4kl58ANX8ap6tqtA9Arcs8uVa1sruE3eaLeLro0ycVAjN%2FhrQ%2FQZtpn1lvl1ofeWkJznAJW0o2rsz001LaPXI6uscIvBXr5R%2FrZvZK%2FMOwn3AZMctRoyYFM1NglTmhzo1g8fwuTEg9gL62J%2BIYkDi95Okml8Ccvdq%2FkKJC0DloHijwMz0urxuU4agAGLXaoi5VZfNjf0YaFT7bZxl01ur%2Bljqz%2FgQFOi8UoGfU0gGvOf5F1T6XaN5HfoLeVA%2BIq9euwJ0E5rPxtCCWBxpzC4UgTApoPfcLZuzgMzcJYvtKzMfHczSVDCwEUhFqkJbISs8hsAO7R4DRauDVAeSgK8MD%2FlWCKl%2FfNXKbrNTyAA&x-oss-date=20251020T065510Z&x-oss-expires=300&x-oss-signature-version=OSS4-HMAC-SHA256&x-oss-credential=STS.NYtFZrni48spybs1jbitpdH2R%2F20251020%2Fap-southeast-1%2Foss%2Faliyun_v4_request&x-oss-signature=b3bf8af1327e626f2402a2ca6d9f937bb35773f71d1efa104b762e8179458f24"

    content = [
        {"type": "text", "text": """
        # 身份
你是一个高精度的【视频内容转录员】。

# 核心任务
你的唯一任务是：以【时间戳】为单位，逐字逐句、逐个动作地记录视频中的所有视听信息。你必须像法庭书记员一样，做到100%的客观和精确。

# 输出格式 (必须严格遵守)
使用以下格式，为每个独立事件创建新的一行：
[HH:MM:SS] [类别]: [内容]

# “类别”包括：
- **[台词]**: 记录人物说的每一句话。如果能识别说话人，格式为 `[台词-人名]`. 如果不能，格式为 `[台词-男/女]`.
- **[画面]**: 描述场景、环境、人物外貌、表情和关键物品。
- **[动作]**: 描述人物的所有具体动作。
- **[音效]**: 记录所有非对话的声音，如(敲门声), (汽车鸣笛)。

# 绝对禁止
- 禁止进行任何总结、分析或评论。
- 禁止使用任何剧本格式。
- 禁止省略任何细节。
- 你的所有输出都必须是简体中文。
        """},
        # {
        #     "type": "video_url",
        #     "video_url": {
        #         "url": url
        #     }}
    ]

    # content = [
    #     {"type": "text", "text": "a dog" * 10000 + "\n\n一共多少个词"},
    # ]

    # content = [
    #     {"type": "text", "text": "一句话总结"},
    #
    #     {
    #         "type": "video_url",
    #         "video_url": {
    #             "url": "https://lmdbk.com/5.mp4",
    #         }
    #     },
    #
    # ]
    # ['qwen3-max', 'qwen3-vl-plus', 'qwen3-coder-plus', 'qwen3-omni-flash']

    from meutils.io.files_utils import to_base64

    request = CompletionRequest(
        # model="qwen3-235b-a22b",
        # model="qwen3-max",
        # model="qwen3-max-2025-10-30-thinking",
        model="qwen3-max-2025-10-30",
        # model="qwen3-max-2025-10-30-search",

        # model="qwen3-vl-plus",
        # model="qwen3-coder-plus",
        # model="qwen3-omni-flash",
        # model="qwen3-235b-a22b-search",

        # model="qwen-image",

        messages=[
            # {
            #     "role": "user",
            #     "content": content,
            # },

            {
                "role": "user",
                "content": "周杰伦",
            },

            # {
            #     "role": "user",
            #     "content": "1+1",
            # },
            # {
            #     "role": "assistant",
            #     "content": "3",
            # },
            # {
            #     "role": "user",
            #     "content": "错了",
            # }
        ],
        stream=True,

        # enable_thinking=True,

        size="1:1"

    )

    logger.debug(request.last_urls)


    async def main():
        response = await Completions(api_key=token, http_client=True).create(request)

        if isinstance(response, AsyncStream):
            async for chunk in response:
                logger.debug(chunk)
        elif isinstance(response, AsyncGenerator):
            async for chunk in response:
                logger.debug(chunk)
        else:
            logger.debug(response)


    #
    #
    # arun(main())

    # image1 = arun(to_base64("https://v3.fal.media/files/penguin/XoW0qavfF-ahg-jX4BMyL_image.webp"))
    #
    request = ImageRequest(
        # model="qwen-image",
        model="qwen-image-edit",

        prompt="把小鸭子放在女人的T恤上面",
        # prompt="裸体女人",

        image=[

            # "https://v3.fal.media/files/penguin/XoW0qavfF-ahg-jX4BMyL_image.webp",
            # "https://v3.fal.media/files/tiger/bml6YA7DWJXOigadvxk75_image.webp"
        ]
    )
    #
    api_key = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjMxMGNiZGFmLTM3NTQtNDYxYy1hM2ZmLTllYzgwMDUzMjljOSIsImxhc3RfcGFzc3dvcmRfY2hhbmdlIjoxNzUwNjYwODczLCJleHAiOjE3NjQxMzY1Mjh9.OqNyJXq0sIS6ocqX0n7_2M2bojLeNAHDBYeYDa_NCgg"
    # api_key = None
    arun(Completions(api_key=api_key).generate(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : fal_types
# @Time         : 2024/11/14 17:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=Gvm9dt"


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : download
# @Time         : 2022/4/6 上午9:48
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 
from meutils.pipe import *
from meutils.request_utils.crawler import Crawler

"""
Crawler(f'https://huggingface.co/datasets/{model_name}/tree/main').xpath("/html/body/div[1]/main/div/section/ul/li[*]/a[1]/span[1]//text()")

(Crawler(f'https://huggingface.co/datasets/{model_name}/tree/main/应用类型识别挑战赛公开数据')
.xpath("/html/body/div[1]/main/div/section/ul/li[*]/a[1]/span[1]//text()"))
"""


def transformers_download(model_name: str, type='torch', out=None):
    """mecli td --model-name ckiplab/albert-tiny-chinese"""

    if out is None:
        out = Path(model_name)
    else:
        out = Path(out) / Path(model_name)

    if out.exists():
        return f'{model_name} already exist'

    out.mkdir(parents=True, exist_ok=True)

    files = Crawler(f'https://huggingface.co/{model_name}/tree/main').xpath("//text()")
    files = [file for file in files if file.endswith(('.txt', '.json', '.py', '.c'))]

    type2name = {
        'torch': 'pytorch_model.bin',
        'tensorflow': 'tf_model.h5',
        'flax': 'flax_model.msgpack'

    }
    if type == 'all':
        model_files = list(type2name.values())
    else:
        model_files = [type2name.get(type, 'torch')]

    files += model_files
    logger.info(files)

    tqdm_ = tqdm(files)
    for file in tqdm_:
        tqdm_.set_description(file)
        url = f"https://huggingface.co/{model_name}/resolve/main/{file}"
        wget.download(url, str(out))


if __name__ == '__main__':
    transformers_download('THUDM/chatglm-6b-int4-qe')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : suno_api
# @Time         : 2025/2/25 14:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# "https://api.chatfire.cn/suno/submit/music"

from openai import AsyncOpenAI
from meutils.schemas.suno_types import SunoAIRequest
from meutils.decorators.retry import retrying


@retrying()  # 触发重试
async def get_task(task_id):  # task_id 实际是 clip_ids， 必须指定token获取任务
    client = AsyncOpenAI(base_url="https://api.chatfire.cn/suno")
    response = await client.get(f"/fetch/{task_id}", cast_to=object)
    return response


async def create_task(request: SunoAIRequest):
    client = AsyncOpenAI(base_url="https://api.chatfire.cn/suno")

    response = await client.post(
        "/submit/music",
        body=request.model_dump(exclude_none=True),
        cast_to=object,

    )

    logger.debug(response)
    return response.get("data")


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : got_ocr
# @Time         : 2024/9/26 18:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 尝试异步任务

from meutils.pipe import *

from meutils.decorators.retry import retrying

from meutils.schemas.ocr_types import OCRRequest
from meutils.io.files_utils import to_tempfile

from meutils.apis.hf.gradio import create_client, handle_file

ENDPOINT = "stepfun-ai/GOT_official_online_demo"

ENDPOINT = "https://s5k.cn/api/v1/studio/stepfun-ai/GOT_official_online_demo/gradio/"


# httpx_kwargs


@retrying(max_retries=3)
async def create(request: OCRRequest):
    client = await create_client(ENDPOINT)

    async with to_tempfile(request.image) as file:
        # client.predict
        job = client.submit(
            image=handle_file(file),
            got_mode=request.mode,
            fine_grained_mode=request.fine_grained_mode,
            ocr_color=request.ocr_color,
            ocr_box=request.ocr_box,
            api_name="/run_GOT",

        )
        text, rendered_html = job.result(timeout=100)
        return text, rendered_html


if __name__ == '__main__':
    url = 'https://oss.ffire.cc/images/vertical-text.jpg'

    # for mode in [
    #     'plain texts OCR', 'plain multi-crop OCR', 'plain fine-grained OCR',
    #     'format texts OCR', 'format multi-crop OCR', 'format fine-grained OCR']:
    #     for i in range(100):
    #         request = OCRRequest(image=url)
    #
    #

    # request = OCRRequest(image=url, mode="format multi-crop OCR")
    # arun(create(request))

    request = OCRRequest(image=url, mode="plain texts OCR")
    texts = arun(create(request))

    print(type(texts))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : niutrans
# @Time         : 2024/4/29 10:55
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import requests

from meutils.pipe import *

import json
import urllib.error
import urllib.parse
import urllib.request
from openai import OpenAI, AsyncOpenAI

base_url = 'http://api.niutrans.com'

client = AsyncOpenAI(base_url=base_url)


@lru_cache()
def _translate(sentence, src_lan, tgt_lan, apikey="f77c833dc48cf93e1e85bea2d6f17459"):
    url = 'http://api.niutrans.com/NiuTransServer/translation?'
    data = {"from": src_lan, "to": tgt_lan, "apikey": apikey, "src_text": sentence}
    data_en = urllib.parse.urlencode(data)
    req = url + "&" + data_en
    res = urllib.request.urlopen(req)
    res = res.read()
    res_dict = json.loads(res)
    if "tgt_text" in res_dict:
        result = res_dict['tgt_text']
    else:
        result = res
    return result


@alru_cache
async def translate(sentence, src_lan, tgt_lan, **kwargs):
    apikey = "f77c833dc48cf93e1e85bea2d6f17459"

    payload = {"from": src_lan, "to": tgt_lan, "apikey": apikey, "src_text": sentence}

    response = await client.post(path="/NiuTransServer/translation", cast_to=object, body=payload)

    logger.debug(response)

    if isinstance(response, str) and "tgt_text" in response:
        data = json.loads(response)
        return data.get('tgt_text')


if __name__ == "__main__":
    # print(translate("你好", 'auto', 'ti'))
    # print(translate("ཁྱོད་བདེ་མོ།", 'auto', 'zh'))
    # print(translate("藏语", 'auto', 'ti'))
    # print(translate("讲个故事吧", 'auto', 'ti'))
    # print(translate("གཏམ་རྒྱུད་ཅིག་བཤད་དང་།", 'auto', 'zh'))

    # in_src = open("zh.txt", "r", encoding='utf-8')
    # out_src = open("zh.txt.big.test", "w", encoding='utf-8')
    # lines = in_src.readlines()
    # for line in lines:
    #     line = line.strip()
    #     trans = translate(line, 'zh', 'en', '您的apikey')
    #     try:
    #         trans = trans.decode('utf-8')
    #     except:
    #         trans = trans
    #     out_src.write(trans + "")

    r = arun(translate("你好", 'auto', 'en'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2024/8/28 10:04
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *
from meutils.notice.feishu import send_message as _send_message
from meutils.db.redis_db import redis_client, redis_aclient
from meutils.config_utils.lark_utils import aget_spreadsheet_values

from meutils.llm.openai_utils import to_openai_completion_params, token_encoder, token_encoder_with_cache
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, ChatCompletionRequest, CompletionUsage
from meutils.schemas.dify_types import BASE_URL, ChatCompletionChunkResponse
from openai import OpenAI, AsyncOpenAI, APIStatusError

send_message = partial(
    _send_message,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e0db85db-0daf-4250-9131-a98d19b909a9",
    title=__name__
)


class Completions(object):

    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key

    async def create(self, request: ChatCompletionRequest):
        payload = {
            "inputs": {},
            "query": request.last_content,
            "response_mode": "streaming",
            "conversation_id": "",
            "user": request.user or 'chatfire',
            "files": [
                {
                    "type": "image",
                    "transfer_method": "remote_url",
                    "url": "https://cloud.dify.ai/logo/logo-site.png"
                }
            ]
        }
        headers = {'Authorization': f'Bearer {self.api_key}'}
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=200) as client:
            async with client.stream(method="POST", url="/chat-messages", json=payload) as response:
                async for chunk in response.aiter_lines():
                    if chunk.startswith("data: {") and (chunk := chunk.strip("data:").strip()):
                        try:
                            chunk_resp = ChatCompletionChunkResponse.model_validate_json(chunk)
                            yield chunk_resp.answer

                            if chunk_resp.event not in {"message", "agent_message"}:  # debug
                                logger.debug(json.loads(chunk))

                        except Exception as e:
                            _ = f"{e}\n{chunk}"
                            logger.error(_)
                            send_message(_)


# data: {"event": "message", "task_id": "900bbd43-dc0b-4383-a372-aa6e6c414227", "id": "663c5084-a254-4040-8ad3-51f2a3c1a77c", "answer": "Hi", "created_at": 1705398420}\n\n
if __name__ == '__main__':
    c = Completions('app-1O2hPvpPf46cmiCjKqd2Dyx5')

    request = ChatCompletionRequest(messages=[{'role': 'user', 'content': '画条狗'}])


    # arun(c.create(request))

    async def main():
        async for i in c.create(request):
            print(i, end='')


    arun(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat_all
# @Time         : 2025/3/4 13:25
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.io.files_utils import to_bytes
from meutils.io.openai_files import file_extract, guess_mime_type
from meutils.str_utils.json_utils import json_path
from meutils.apis.search import metaso
# from meutils.apis.chatglm import glm_video_api

from meutils.llm.clients import chatfire_client, zhipuai_client, AsyncOpenAI
from meutils.llm.openai_utils import to_openai_params

from meutils.schemas.openai_types import ChatCompletionRequest
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, ImageRequest

from fake_useragent import UserAgent

ua = UserAgent()

"""
todo: ppt

"""


class Completions(object):

    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key
        self.client = AsyncOpenAI(api_key=api_key)

    async def create(self, request: CompletionRequest):
        """
        :param request:
        :return:
        """
        request.model = request.model.removesuffix("-all").removesuffix("-plus")  ### 避免死循环

        if request.last_user_content.startswith(("画",)):  # 画画
            return await self.create_images(request)  # str

        # ppt 视频生成
        # elif request.last_user_content.startswith(("联网", "搜索", "在线搜索", "在线查询")):  # 画画
        #     return metaso.create(request)
        # elif request.last_user_content.startswith(("视频",)):  # 视频生成

        elif request.last_user_content.startswith(("联网", "搜索", "在线搜索", "在线查询")):  # 画画
            return metaso.create(request)

        elif request.last_user_content.startswith(("http",)):

            file_url, *texts = request.last_user_content.split(maxsplit=1) + ["总结下"]  # application/octet-stream
            text = texts[0]

            if guess_mime_type(file_url).startswith("image"):  # 识图
                request.model = "glm-4v-flash"
                request.messages = [
                    {
                        'role': 'user',
                        'content': [
                            {
                                "type": "text",
                                "text": text
                            },

                            {
                                "type": "image_url",
                                "image_url": {
                                    "url": file_url
                                }
                            }
                        ]
                    }
                ]
                data = to_openai_params(request)
                return await zhipuai_client.chat.completions.create(**data)

            elif guess_mime_type(file_url).startswith(("video", "audio")):  # 音频 视频
                request.model = "gemini"  # 果果
                request.messages = [
                    {
                        'role': 'user',
                        'content': [
                            {
                                "type": "text",
                                "text": text
                            },
                            {
                                "type": "image_url",
                                "image_url": {
                                    "url": file_url
                                }
                            }
                        ]
                    }
                ]
                data = to_openai_params(request)
                return await self.client.chat.completions.create(**data)

            else:
                # logger.debug(f"file_url: {file_url}")
                file_content = await file_extract(file_url)  # 文件问答-单轮

                logger.debug(file_content)

                request.messages = [
                    {
                        'role': 'user',
                        'content': f"""{json.dumps(file_content, ensure_ascii=False)}\n\n{text}"""
                    }
                ]
                # logger.debug(request)
                data = to_openai_params(request)
                return await self.client.chat.completions.create(**data)

        if image_urls := request.last_urls.get("image_url"):  # 识图
            request.model = "glm-4v-flash"
            data = to_openai_params(request)
            return await zhipuai_client.chat.completions.create(**data)

        elif file_urls := request.last_urls.get("file_url"):
            return await self.chat_files(request)

        # todo 标准格式的audio_url video_url

        data = to_openai_params(request)
        return await self.client.chat.completions.create(**data)

    async def chat_files(self, request: CompletionRequest):  # 多轮
        for i, message in enumerate(request.messages[::-1], 1):
            if message.get("role") == "user":
                texts = json_path(message, expr='$..text') or [""]
                file_urls = json_path(message, expr='$..file_url.url') or [""]

                logger.debug(f"""{texts} \n\n {file_urls}""")

                text, file_url = texts[-1], file_urls[-1]
                if file_url in request.last_urls.get("file_url", []):
                    file_content = await file_extract(file_url)

                    message["content"] = f"""{json.dumps(file_content, ensure_ascii=False)}\n\n{text}"""

                    request.messages = request.messages[-i:]
                    break  # 截断：从最新的文件开始

        data = to_openai_params(request)
        return await self.client.chat.completions.create(**data)

    async def create_images(self, request: CompletionRequest):

        response = await zhipuai_client.images.generate(
            model="cogview-3-flash",
            prompt=request.last_user_content,
            n=1
        )
        image_url = response.data[0].url
        tool_desc = """> images.generate\n\n"""
        tool_desc += f"![{request.last_user_content}]({image_url})"
        if not request.stream:
            chat_completion.choices[0].message.content = tool_desc
            return chat_completion
        return tool_desc

    # chat_completion_chunk

    # async def create_videos(self, request: CompletionRequest):
    #
    #     request = ImageRequest(prompt=request.last_user_content)
    #
    #     response = await glm_video_api.generate(request)
    #     image_url = response.data[0].url
    #     tool_desc = """> images.generate\n\n"""
    #     return tool_desc + f"![{request.last_user_content}]({image_url})"


if __name__ == '__main__':
    c = Completions()

    request = CompletionRequest(
        # model="qwen-turbo-2024-11-01",
        # model="claude-3-5-sonnet-20241022",
        model="gpt-4o-mini",

        messages=[{
            'role': 'user',
            'content': [
                {
                    "type": "text",
                    # "text": "总结下"
                    "text": "https://app.yinxiang.com/fx/8b8bba1e-b254-40ff-81e1-fa3427429efe 总结下",

                    # "text": "https://app.yinxiang.com/fx/8b8bba1e-b254-40ff-81e1-fa3427429efe"

                },

                # {
                #     "type": "file_url",
                #     "file_url": {
                #         "url": "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf"
                #     }
                # }
            ]
        }])

    arun(c.create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : prodia_types
# @Time         : 2024/8/14 14:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

BASE_URL = "https://api.prodia.com/v1"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=71b08f"


class FaceswapRequest(BaseModel):
    sourceUrl: str
    targetUrl: str



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ollama_search
# @Time         : 2025/9/26 09:00
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://docs.ollama.com/web-search

from meutils.pipe import *

"""
curl https://ollama.com/api/web_search \
  --header "Authorization: Bearer $OLLAMA_API_KEY" \
	-d '{
	  "query":"what is ollama?"
	}'
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_images
# @Time         : 2024/10/16 08:54
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from openai import AsyncOpenAI
from openai import APIStatusError

from meutils.pipe import *
from meutils.io.files_utils import to_base64, to_url
from meutils.llm.openai_utils import to_openai_images_params
from meutils.apis.utils import create_http_client
from meutils.notice.feishu import IMAGES, send_message as _send_message

from meutils.schemas.image_types import ImageRequest, FluxImageRequest, SDImageRequest, ImagesResponse

send_message = partial(
    _send_message,
    title=__name__,
    url=IMAGES
)

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=3aA5dH"
FEISHU_URL_FREE = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=xlvlrH"

BASE_URL = "https://api.siliconflow.cn/v1"


async def generate(request: ImageRequest, api_key: Optional[str] = None, base_url: Optional[str] = None):
    base_url = base_url or BASE_URL

    request.prompt_enhancement = True
    if request.model.lower().startswith("qwen"):
        if request.image:
            request.model = "Qwen/Qwen-Image-Edit-2509"

            for k, v in zip(["image", "image2", "image3"], request.image):
                setattr(request, k, v)

    data = to_openai_images_params(request)
    if len(str(data)) < 1000: logger.debug(bjson(data))

    try:
        client = AsyncOpenAI(base_url=base_url, api_key=api_key)
        response = await client.images.generate(**data)
        # logger.debug(response)

    except APIStatusError as e:
        logger.debug(e)
        # logger.debug(e.response.json())
        # logger.debug(e.response.status_code)

        if e.response.status_code > 403 and any(i in BASE_URL for i in {"siliconflow", "modelscope"}):
            client = AsyncOpenAI(
                base_url=BASE_URL,
                api_key=api_key,
                http_client=await create_http_client(True)
            )
            response = await client.images.generate(**data)
        raise e

    return response


if __name__ == '__main__':
    from meutils.pipe import *

    api_key = os.getenv("SILICONFLOW_API_KEY")

    data = {
        "model": "Qwen/Qwen-Image",
        # "model": "black-forest-labs/FLUX.1-Krea-dev",

        "prompt": "A young woman and a monkey inside a colorful house",
        "image": [
            "https://v3.fal.media/files/panda/HDpZj0eLjWwCpjA5__0l1_0e6cd0b9eb7a4a968c0019a4eee15e46.png",
            "https://v3.fal.media/files/zebra/153izt1cBlMU-TwD0_B7Q_ea34618f5d974653a16a755aa61e488a.png",
            "https://v3.fal.media/files/koala/RCSZ7VEEKGFDfMoGHCwzo_f626718793e94769b1ad36d5891864a4.png"
        ],
        "aspect_ratio": "16:9",
        "movement_amplitude": "auto"
    }

    arun(generate(ImageRequest(**data), api_key))

    # https://api.siliconflow.cn/v1



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ocr
# @Time         : 2023/5/18 16:11
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/RapidAI/RapidOCR/wiki/RapidOCR%E8%B0%83%E4%BC%98%E5%B0%9D%E8%AF%95%E6%95%99%E7%A8%8B

# LocalOcr

from meutils.pipe import *
from rapidocr_onnxruntime import RapidOCR

# from rapidocr_openvino import RapidOCR

rapid_ocr = RapidOCR()
# results, elapse = rapid_ocr('invoice.jpg')
# results, elapse = rapid_ocr('tbl.png')
results, elapse = rapid_ocr('bg.png')

rprint(results)
print([i[1] for i in results])

# rprint([dict(zip(['坐标', '文字'], r)) for r in results])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2025/4/15 12:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
#!/usr/bin/env Python
# -*- coding: utf-8 -*-

"""
使用requests请求代理服务器
请求http和https网页均适用
"""

import random
import asyncio

import httpx
import requests


# API接口，返回格式为json
api_url = "https://dps.kdlapi.com/api/getdps/?secret_id=ot71jj2r4w9coubcip9o&signature=dldk7s859lixl44qxcg7idw8qocy6hrm&num=1&pt=1&format=json&sep=1"  # API接口


secret_id ="owklc8tk3ypo00ohu80o"
signature = "8gqqy7w64g7uunseaz9tcae7h8saa24p"
api_url = f"https://dps.kdlapi.com/api/getdps/?secret_id={secret_id}&signature={signature}&num=1&pt=1&format=json&sep=1"

# API接口返回的proxy_list
proxy_list = requests.get(api_url).json().get('data').get('proxy_list')

# 用户名密码认证(私密代理/独享代理)
username = "d1999983904"
password = "1h29rymg"


page_url = "http://icanhazip.com/"  # 要访问的目标网页


async def fetch(url):
    proxy = httpx.Proxy(
        url=f"http://{username}:{password}@{random.choice(proxy_list)}",
    )
    async with httpx.AsyncClient(proxy=proxy, timeout=10) as client:
        resp = await client.get(url)
        print(f"status_code: {resp.status_code}, content: {resp.content}")


def run():
    loop = asyncio.get_event_loop()
    # 异步发出5次请求
    tasks = [fetch(page_url) for _ in range(5)]
    loop.run_until_complete(asyncio.wait(tasks))


if __name__ == '__main__':
    run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/8/23 10:26
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.decorators.retry import retrying
from meutils.oss.minio_oss import Minio
from meutils.pipe import *

from meutils.schemas.openai_types import ImageRequest, ImagesResponse
from meutils.schemas.image_types import ASPECT_RATIOS

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=IN01qU"

from urllib.parse import urlencode

STYLES = {'AUTO', 'DEFAULT', 'PHOTO', 'ANIME', 'ILLUSTRATION', 'RENDER_3D'}


# @alru_cache(ttl=600)
# @retrying()
async def get_access_token(refresh_token: Optional[str] = None):
    refresh_token = refresh_token or await get_next_token_for_polling(FEISHU_URL)

    refresh_token, cookie = refresh_token.strip().split('\n', maxsplit=1)

    payload = {
        "grant_type": "refresh_token",
        "refresh_token": refresh_token,
    }

    data = urlencode(payload)

    headers = {
        'accept': '*/*',
        'accept-language': 'zh-CN,zh;q=0.9',
        'content-type': 'application/x-www-form-urlencoded',
        'origin': 'https://ideogram.ai',
        'priority': 'u=1, i',
        'referer': 'https://ideogram.ai/',
        'sec-ch-ua': '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"macOS"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'cross-site',
        'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36',
        # 'x-client-data': 'CI22yQEIo7bJAQipncoBCNqFywEIlKHLAQiGoM0BCJ2szgEYj87NAQ==',
        'x-client-version': 'Chrome/JsCore/10.12.3/FirebaseCore-web'
    }

    base_url = "https://ideogram.chatfire.cc"
    base_url = "https://securetoken.googleapis.com"
    params = {"key": "AIzaSyBwq4bRiOapXYaKE-0Y46vLAw1-fzALq7Y"}
    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=30) as client:
        response = await client.post('/v1/token', content=data, params=params)
        # logger.debug(response.status_code)
        # logger.debug(response.text)
        data = response.json()

        user_id = data['user_id']
        access_token = data['access_token']

        return user_id, access_token, cookie


@retrying(max_retries=8, max=5, predicate=lambda x: x is True)
async def create(request: ImageRequest, refresh_token: Optional[str] = None):
    user_id, access_token, cookie = await get_access_token(refresh_token)

    width, height = ASPECT_RATIOS.get(request.size, request.size).split('x') | xmap(int)

    payload = {
        "sampling_speed": 2,  # 2 是快速 0 是中速 -2 高质量

        "prompt": request.prompt,
        "user_id": "",
        # "model_version": "V_1_5",
        # "model_version": "V_0_3",
        "model_version": request.model,

        "use_autoprompt_option": "AUTO",
        "style_expert": request.style if request.style in STYLES else "AUTO",

        "resolution": {
            "width": width,
            "height": height
        },
        # "resolution": {
        #     "width": 1152,
        #     "height": 864
        # },
        # "resolution": {
        #     "width": 1280,
        #     "height": 960
        # },

        # "resolution": {
        #     "width": 1024,
        #     "height": 512
        # }
    }

    headers = {
        "Authorization": f"Bearer {access_token}",
        "user_id": user_id,

        "Cookie": cookie,

        "Host": "ideogram.ai",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:125.0) Gecko/20100101 Firefox/125.0",
        "Accept": "*/*",
        "Accept-Language": "en-US,en;q=0.5",
        "Accept-Encoding": "gzip, deflate, br",
        "Referer": "https://ideogram.ai/u/test1014/generated",
        "Content-Type": "application/json",
        "Alt-Used": "ideogram.ai",
        "Connection": "keep-alive",
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "TE": "trailers"
    }

    base_url = "https://ideogram.ai"
    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=30) as client:
        response = await client.post('/api/images/sample', json=payload)
        logger.debug(response.status_code)
        # logger.debug(response.text)
        if response.status_code in {401, 403}:
            return True

        if response.is_success:
            data = response.json()
            payload = {
                "request_ids": [data['request_id']]
            }

            for i in range(10):
                await asyncio.sleep(5)
                logger.debug(f'获取结果 「{i}」')
                response = await client.post("/api/gallery/retrieve-requests", json=payload)

                logger.debug(response.status_code)
                # logger.debug(response.text)

                if response.is_success:
                    data = response.json()
                    if data['sampling_requests'][0]['is_completed']:
                        image_responses = data['sampling_requests'][0]['responses']
                        tasks = []
                        for image_resp in image_responses:
                            url = f"https://ideogram.ai/api/images/direct/{image_resp['response_id']}"

                            image_resp['raw_url'] = url

                            task = await Minio().put_object_for_openai(
                                file=url,
                                filename=f"{shortuuid.random()}.jpeg",

                                headers=headers, follow_redirects=True
                            )

                            tasks.append(task)
                            await asyncio.sleep(1)

                        # file_objects = await asyncio.gather(*tasks)

                        file_objects = tasks
                        for file_object, image_resp in zip(file_objects, image_responses):
                            image_resp['url'] = file_object.filename

                        return ImagesResponse.construct(data=image_responses)


if __name__ == '__main__':
    # arun(get_access_token(refresh_token))  # 88uwVq1nbyctFsgAOWytFefTAW02
    with timer():
        arun(
            create(
                ImageRequest(
                    model='V_0_3',

                    prompt='一位性感的中国美女',

                    style=''
                ))
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : sophnet
# @Time         : 2025/6/4 16:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.clients import AsyncOpenAI

from meutils.llm.check_utils import check_token_for_sophnet as check_token

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, CompletionUsage, \
    ChatCompletion

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=lxJ27j"

base_url = "https://www.sophnet.com/api/open-apis/v1"


async def create(request: CompletionRequest, token: Optional[str] = None):
    """最终是非流"""
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True, check_token=check_token)

    client = AsyncOpenAI(base_url=base_url, api_key=token)

    if not request.stream:
        logger.debug("伪非流")

        data = request.model_dump(exclude_none=True)
        data['stream'] = True

        chunks = await client.chat.completions.create(**data)

        chunk = None
        async for chunk in chunks:
            if chunk.choices:
                _ = chunk.choices[0].delta.content or ""
                chat_completion.choices[0].message.content += _
                # logger.debug(_)
        if hasattr(chunk, "usage"):
            chat_completion.usage = chunk.usage

        logger.debug(chat_completion)
        return chat_completion
    else:
        return client.chat.completions.create(**request.model_dump(exclude_none=True))


if __name__ == '__main__':
    request = CompletionRequest(
        model="DeepSeek-v3",
        messages=[{"role": "user", "content": "hi"}],
        stream=True,
    )

    arun(create(request))



"""Top-level package for {{ cookiecutter.project_name }}."""
import time

__author__ = """{{ cookiecutter.full_name }}"""
__email__ = '{{ cookiecutter.email }}'
# __version__ = '{{ cookiecutter.version }}'
__version__ = time.strftime("%Y.%m.%d.%H.%M.%S", time.localtime())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2024/9/26 15:54
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from meutils.decorators.retry import retrying

url = "https://api.siliconflow.cn/v1/user/info"


# todo: 付费不付费模型优化
@retrying()
async def check_token(api_key):
    headers = {
        "Authorization": f"Bearer {api_key}",
        "Accept": "application/json"
    }
    try:
        async with httpx.AsyncClient(headers=headers, timeout=60) as client:
            response: httpx.Response = await client.get(url)
            response.raise_for_status()

            logger.debug(response.text)
            logger.debug(response.status_code)

            if response.is_success:
                balance = response.json()['data']['balance']
                return float(balance) > 0
    except Exception as e:
        logger.error(e)
        return False


if __name__ == '__main__':
    from meutils.pipe import *
    from openai import OpenAI

    api_key = os.getenv("SILICONFLOW_API_KEY")  # sk-fbumsiflwnsgdvegjksroqsgejocgigbnvltqiwhrasixnra

    feishu_url = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=InxiCF"
    feishu_url = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=GSHr1U"
    from meutils.config_utils.lark_utils import aget_spreadsheet_values

    df = arun(aget_spreadsheet_values(feishu_url=feishu_url, to_dataframe=True))

    for i in df[0]:
        if i:
            try:
                r = OpenAI(api_key=i,
                           # base_url=os.getenv("SILICONFLOW_BASE_URL"),
                           base_url="http://siliconflow.ffire.cc/v1"
                           ).chat.completions.create(
                    messages=[{"role": "user", "content": "你是谁"}], model="Qwen/Qwen2.5-7B-Instruct")
                print(r)
            except Exception as e:
                logger.error(e)
                print(i)





from meutils.pipe import *
from meutils.hash_utils import *

key = '1eb25317-39a1-4af7-a6e3-63877ec2dd64'
hook_url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={key}'  # 发送消息接口地址


def wx_post(file='wechat.py'):
    id_url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/upload_media?key={key}&type=file'  # 上传文件接口地址
    data = {'file': open(file, 'rb')}  # post jason
    response = requests.post(url=id_url, files=data)  # post 请求上传文件
    json_res = response.json()  # 返回转为json
    print(json_res)
    media_id = json_res['media_id']  # 提取返回ID

    wx_url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={key}'  # 发送消息接口地址
    data = {"msgtype": "file", "file": {"media_id": media_id}}  # post json
    r = requests.post(url=wx_url, json=data)  # post请求消息
    return r  # 返回请求状态



if __name__ == '__main__':
    print(wx_post().text)